@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,6 +38,7 @@ import org.ojalgo.function.VoidFunction;
 import org.ojalgo.function.aggregator.Aggregator;
 import org.ojalgo.function.aggregator.AggregatorFunction;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.scalar.Scalar;
@@ -46,6 +47,7 @@ import org.ojalgo.structure.Factory1D;
 import org.ojalgo.structure.Mutate1D;
 import org.ojalgo.structure.Transformation1D;
 import org.ojalgo.tensor.TensorFactory1D;
+import org.ojalgo.type.math.MathType;
 
 /**
  * Array1D
@@ -65,18 +67,22 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
             myDelegate = new BasicArray.Factory<>(denseArray);
         }
 
+        @Override
         public Array1D<N> copy(final Access1D<?> source) {
             return myDelegate.copy(source).wrapInArray1D();
         }
 
-        public Array1D<N> copy(final Comparable<?>... source) {
+        @Override
+        public Array1D<N> copy(final Comparable<?>[] source) {
             return myDelegate.copy(source).wrapInArray1D();
         }
 
+        @Override
         public Array1D<N> copy(final double... source) {
             return myDelegate.copy(source).wrapInArray1D();
         }
 
+        @Override
         public Array1D<N> copy(final List<? extends Comparable<?>> source) {
             return myDelegate.copy(source).wrapInArray1D();
         }
@@ -86,19 +92,27 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
             return myDelegate.function();
         }
 
+        @Override
+        public MathType getMathType() {
+            return myDelegate.getMathType();
+        }
+
         @Override
         public Array1D<N> make(final long count) {
             return this.makeDense(count);
         }
 
+        @Override
         public Array1D<N> makeDense(final long count) {
             return myDelegate.makeToBeFilled(count).wrapInArray1D();
         }
 
+        @Override
         public Array1D<N> makeFilled(final long count, final NullaryFunction<?> supplier) {
             return myDelegate.makeFilled(count, supplier).wrapInArray1D();
         }
 
+        @Override
         public Array1D<N> makeSparse(final long count) {
             return myDelegate.makeStructuredZero(count).wrapInArray1D();
         }
@@ -244,27 +258,16 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
 
     }
 
-    public static final Factory<ComplexNumber> C128 = Array1D.factory(ArrayC128.FACTORY);
-    public static final Factory<Quaternion> H256 = Array1D.factory(ArrayH256.FACTORY);
-    public static final Factory<RationalNumber> Q128 = Array1D.factory(ArrayQ128.FACTORY);
     public static final Factory<Double> R032 = Array1D.factory(ArrayR032.FACTORY);
     public static final Factory<Double> R064 = Array1D.factory(ArrayR064.FACTORY);
-    public static final Factory<BigDecimal> R128 = Array1D.factory(ArrayR128.FACTORY);
-    public static final Factory<Double> Z008 = Array1D.factory(ArrayZ008.FACTORY);
-    public static final Factory<Double> Z016 = Array1D.factory(ArrayZ016.FACTORY);
-    public static final Factory<Double> Z032 = Array1D.factory(ArrayZ032.FACTORY);
-    public static final Factory<Double> Z064 = Array1D.factory(ArrayZ064.FACTORY);
-
+    public static final Factory<Quadruple> R128 = Array1D.factory(ArrayR128.FACTORY);
+    public static final Factory<BigDecimal> R256 = Array1D.factory(ArrayR256.FACTORY);
     /**
-     * @deprecated v52 Use {@link #Q128} instead
+     * @deprecated v52 Use {@link #R256} instead
      */
     @Deprecated
-    public static final Factory<RationalNumber> RATIONAL = Q128;
-    /**
-     * @deprecated v52 Use {@link #R128} instead
-     */
-    @Deprecated
-    public static final Factory<BigDecimal> BIG = R128;
+    public static final Factory<BigDecimal> BIG = R256;
+    public static final Factory<ComplexNumber> C128 = Array1D.factory(ArrayC128.FACTORY);
     /**
      * @deprecated v52 Use {@link #C128} instead
      */
@@ -280,6 +283,7 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
      */
     @Deprecated
     public static final Factory<Double> DIRECT64 = Array1D.factory(BufferArray.DIRECT64);
+    public static final Factory<Quaternion> H256 = Array1D.factory(ArrayH256.FACTORY);
     /**
      * @deprecated v52 Use {@link #R032} instead
      */
@@ -290,11 +294,21 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
      */
     @Deprecated
     public static final Factory<Double> PRIMITIVE64 = R064;
+    public static final Factory<RationalNumber> Q128 = Array1D.factory(ArrayQ128.FACTORY);
     /**
      * @deprecated v52 Use {@link #H256} instead
      */
     @Deprecated
     public static final Factory<Quaternion> QUATERNION = H256;
+    /**
+     * @deprecated v52 Use {@link #Q128} instead
+     */
+    @Deprecated
+    public static final Factory<RationalNumber> RATIONAL = Q128;
+    public static final Factory<Double> Z008 = Array1D.factory(ArrayZ008.FACTORY);
+    public static final Factory<Double> Z016 = Array1D.factory(ArrayZ016.FACTORY);
+    public static final Factory<Double> Z032 = Array1D.factory(ArrayZ032.FACTORY);
+    public static final Factory<Double> Z064 = Array1D.factory(ArrayZ064.FACTORY);
 
     public static <N extends Comparable<N>> Array1D.Factory<N> factory(final DenseArray.Factory<N> denseFactory) {
         return new Array1D.Factory<>(denseFactory);
@@ -366,6 +380,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return visitor.get();
     }
 
+    @Override
+    public byte byteValue(final int index) {
+        return myDelegate.byteValue(this.convert(index));
+    }
+
     @Override
     public byte byteValue(final long index) {
         return myDelegate.byteValue(this.convert(index));
@@ -386,6 +405,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return length;
     }
 
+    @Override
+    public double doubleValue(final int index) {
+        return myDelegate.doubleValue(this.convert(index));
+    }
+
     @Override
     public double doubleValue(final long index) {
         return myDelegate.doubleValue(this.convert(index));
@@ -448,6 +472,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         myDelegate.fill(this.convert(first), this.convert(limit), myStep, supplier);
     }
 
+    @Override
+    public float floatValue(final int index) {
+        return myDelegate.floatValue(this.convert(index));
+    }
+
     @Override
     public float floatValue(final long index) {
         return myDelegate.floatValue(this.convert(index));
@@ -501,6 +530,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return (myDelegate.indexOfLargest(first, limit, step) - first) / step;
     }
 
+    @Override
+    public int intValue(final int index) {
+        return myDelegate.intValue(this.convert(index));
+    }
+
     @Override
     public int intValue(final long index) {
         return myDelegate.intValue(this.convert(index));
@@ -511,6 +545,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return length == 0;
     }
 
+    @Override
+    public long longValue(final int index) {
+        return myDelegate.longValue(this.convert(index));
+    }
+
     @Override
     public long longValue(final long index) {
         return myDelegate.longValue(this.convert(index));
@@ -586,10 +625,36 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         myDelegate.modify(this.convert(first), this.convert(limit), myStep, modifier);
     }
 
+    @Override
     public void reset() {
         myDelegate.reset();
     }
 
+    @Override
+    public void set(final int index, final byte value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
+    @Override
+    public void set(final int index, final double value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
+    @Override
+    public void set(final int index, final float value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
+    @Override
+    public void set(final int index, final int value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
+    @Override
+    public void set(final int index, final long value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
     @Override
     public N set(final int index, final N value) {
         long tmpIndex = this.convert(index);
@@ -598,6 +663,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return retVal;
     }
 
+    @Override
+    public void set(final int index, final short value) {
+        myDelegate.set(this.convert(index), value);
+    }
+
     @Override
     public void set(final long index, final byte value) {
         myDelegate.set(this.convert(index), value);
@@ -633,6 +703,11 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         myDelegate.set(this.convert(index), value);
     }
 
+    @Override
+    public short shortValue(final int index) {
+        return myDelegate.shortValue(this.convert(index));
+    }
+
     @Override
     public short shortValue(final long index) {
         return myDelegate.shortValue(this.convert(index));
@@ -691,6 +766,7 @@ public final class Array1D<N extends Comparable<N>> extends AbstractList<N> impl
         return this.sliceRange(first, limit);
     }
 
+    @Override
     public void supplyTo(final Mutate1D receiver) {
         long limit = Math.min(length, receiver.count());
         if (myDelegate.isPrimitive()) {
