@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,7 @@ import org.ojalgo.function.VoidFunction;
 import org.ojalgo.function.aggregator.Aggregator;
 import org.ojalgo.function.aggregator.AggregatorFunction;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.scalar.Scalar;
@@ -43,6 +44,7 @@ import org.ojalgo.structure.Mutate2D;
 import org.ojalgo.structure.Structure2D;
 import org.ojalgo.structure.Transformation2D;
 import org.ojalgo.tensor.TensorFactory2D;
+import org.ojalgo.type.math.MathType;
 
 /**
  * Array2D
@@ -62,6 +64,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             myDelegate = new BasicArray.Factory<>(denseArray);
         }
 
+        @Override
         public Array2D<N> columns(final Access1D<?>... source) {
 
             int tmpColumns = source.length;
@@ -90,6 +93,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> columns(final Comparable<?>[]... source) {
 
             int tmpColumns = source.length;
@@ -108,6 +112,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> columns(final double[]... source) {
 
             int tmpColumns = source.length;
@@ -126,6 +131,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> columns(final List<? extends Comparable<?>>... source) {
 
             int tmpColumns = source.length;
@@ -144,6 +150,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> copy(final Access2D<?> source) {
             return myDelegate.copy(source).wrapInArray2D(source.countRows());
         }
@@ -153,15 +160,22 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return myDelegate.function();
         }
 
+        @Override
+        public MathType getMathType() {
+            return myDelegate.getMathType();
+        }
+
         @Override
         public Array2D<N> make(final long rows, final long columns) {
             return this.makeDense(rows, columns);
         }
 
+        @Override
         public Array2D<N> makeDense(final long rows, final long columns) {
             return myDelegate.makeToBeFilled(rows, columns).wrapInArray2D(rows);
         }
 
+        @Override
         public Array2D<N> makeFilled(final long rows, final long columns, final NullaryFunction<?> supplier) {
 
             BasicArray<N> tmpDelegate = myDelegate.makeToBeFilled(rows, columns);
@@ -176,10 +190,12 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(rows);
         }
 
+        @Override
         public Array2D<N> makeSparse(final long rows, final long columns) {
             return myDelegate.makeStructuredZero(rows, columns).wrapInArray2D(rows);
         }
 
+        @Override
         public Array2D<N> rows(final Access1D<?>... source) {
 
             int tmpRows = source.length;
@@ -206,6 +222,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> rows(final Comparable<?>[]... source) {
 
             int tmpRows = source.length;
@@ -223,6 +240,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> rows(final double[]... source) {
 
             int tmpRows = source.length;
@@ -240,6 +258,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
             return tmpDelegate.wrapInArray2D(tmpRows);
         }
 
+        @Override
         public Array2D<N> rows(final List<? extends Comparable<?>>... source) {
 
             int tmpRows = source.length;
@@ -268,14 +287,57 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
 
     }
 
-    public static final Factory<BigDecimal> BIG = new Factory<>(BigArray.FACTORY);
-    public static final Factory<ComplexNumber> COMPLEX = new Factory<>(ComplexArray.FACTORY);
-    public static final Factory<Double> DIRECT32 = new Factory<>(BufferArray.DIRECT32);
-    public static final Factory<Double> DIRECT64 = new Factory<>(BufferArray.DIRECT64);
-    public static final Factory<Double> PRIMITIVE32 = new Factory<>(Primitive32Array.FACTORY);
-    public static final Factory<Double> PRIMITIVE64 = new Factory<>(Primitive64Array.FACTORY);
-    public static final Factory<Quaternion> QUATERNION = new Factory<>(QuaternionArray.FACTORY);
-    public static final Factory<RationalNumber> RATIONAL = new Factory<>(RationalArray.FACTORY);
+    public static final Factory<Double> R032 = Array2D.factory(ArrayR032.FACTORY);
+    public static final Factory<Double> R064 = Array2D.factory(ArrayR064.FACTORY);
+    public static final Factory<Quadruple> R128 = Array2D.factory(ArrayR128.FACTORY);
+    public static final Factory<BigDecimal> R256 = Array2D.factory(ArrayR256.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #R256} instead
+     */
+    @Deprecated
+    public static final Factory<BigDecimal> BIG = R256;
+    public static final Factory<ComplexNumber> C128 = Array2D.factory(ArrayC128.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #C128} instead
+     */
+    @Deprecated
+    public static final Factory<ComplexNumber> COMPLEX = C128;
+    /**
+     * @deprecated v52 Use {@link #factory(DenseArray.Factory)} instead
+     */
+    @Deprecated
+    public static final Factory<Double> DIRECT32 = Array2D.factory(BufferArray.DIRECT32);
+    /**
+     * @deprecated v52 Use {@link #factory(DenseArray.Factory)} instead
+     */
+    @Deprecated
+    public static final Factory<Double> DIRECT64 = Array2D.factory(BufferArray.DIRECT64);
+    public static final Factory<Quaternion> H256 = Array2D.factory(ArrayH256.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #R032} instead
+     */
+    @Deprecated
+    public static final Factory<Double> PRIMITIVE32 = R032;
+    /**
+     * @deprecated v52 Use {@link #R064} instead
+     */
+    @Deprecated
+    public static final Factory<Double> PRIMITIVE64 = R064;
+    public static final Factory<RationalNumber> Q128 = Array2D.factory(ArrayQ128.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #H256} instead
+     */
+    @Deprecated
+    public static final Factory<Quaternion> QUATERNION = H256;
+    /**
+     * @deprecated v52 Use {@link #Q128} instead
+     */
+    @Deprecated
+    public static final Factory<RationalNumber> RATIONAL = Q128;
+    public static final Factory<Double> Z008 = Array2D.factory(ArrayZ008.FACTORY);
+    public static final Factory<Double> Z016 = Array2D.factory(ArrayZ016.FACTORY);
+    public static final Factory<Double> Z032 = Array2D.factory(ArrayZ032.FACTORY);
+    public static final Factory<Double> Z064 = Array2D.factory(ArrayZ064.FACTORY);
 
     public static <N extends Comparable<N>> Array2D.Factory<N> factory(final DenseArray.Factory<N> denseArray) {
         return new Array2D.Factory<>(denseArray);
@@ -295,6 +357,11 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         myColumnsCount = structure == 0L ? 0L : delegate.count() / structure;
     }
 
+    @Override
+    public void add(final long index, final byte addend) {
+        myDelegate.add(index, addend);
+    }
+
     @Override
     public void add(final long index, final Comparable<?> addend) {
         myDelegate.add(index, addend);
@@ -310,6 +377,21 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         myDelegate.add(index, addend);
     }
 
+    @Override
+    public void add(final long index, final int addend) {
+        myDelegate.add(index, addend);
+    }
+
+    @Override
+    public void add(final long index, final long addend) {
+        myDelegate.add(index, addend);
+    }
+
+    @Override
+    public void add(final long row, final long col, final byte addend) {
+        myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
+    }
+
     @Override
     public void add(final long row, final long col, final Comparable<?> addend) {
         myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
@@ -325,6 +407,26 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
     }
 
+    @Override
+    public void add(final long row, final long col, final int addend) {
+        myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
+    }
+
+    @Override
+    public void add(final long row, final long col, final long addend) {
+        myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
+    }
+
+    @Override
+    public void add(final long row, final long col, final short addend) {
+        myDelegate.add(Structure2D.index(myRowsCount, row, col), addend);
+    }
+
+    @Override
+    public void add(final long index, final short addend) {
+        myDelegate.add(index, addend);
+    }
+
     @Override
     public N aggregateColumn(final long row, final long col, final Aggregator aggregator) {
         AggregatorFunction<N> visitor = aggregator.getFunction(myDelegate.factory().aggregator());
@@ -353,6 +455,26 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return visitor.get();
     }
 
+    @Override
+    public byte byteValue(final int index) {
+        return myDelegate.byteValue(index);
+    }
+
+    @Override
+    public byte byteValue(final int row, final int col) {
+        return myDelegate.byteValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public byte byteValue(final long index) {
+        return myDelegate.byteValue(index);
+    }
+
+    @Override
+    public byte byteValue(final long row, final long col) {
+        return myDelegate.byteValue(Structure2D.index(myRowsCount, row, col));
+    }
+
     @Override
     public long count() {
         return myDelegate.count();
@@ -368,6 +490,16 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return myRowsCount;
     }
 
+    @Override
+    public double doubleValue(final int index) {
+        return myDelegate.doubleValue(index);
+    }
+
+    @Override
+    public double doubleValue(final int row, final int col) {
+        return myDelegate.doubleValue(Structure2D.index(myRowsCount, row, col));
+    }
+
     @Override
     public double doubleValue(final long index) {
         return myDelegate.doubleValue(index);
@@ -532,10 +664,31 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
      *
      * @see org.ojalgo.structure.Structure2D.Reshapable#flatten()
      */
+    @Override
     public Array1D<N> flatten() {
         return myDelegate.wrapInArray1D();
     }
 
+    @Override
+    public float floatValue(final int index) {
+        return myDelegate.floatValue(index);
+    }
+
+    @Override
+    public float floatValue(final int row, final int col) {
+        return myDelegate.floatValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public float floatValue(final long index) {
+        return myDelegate.floatValue(index);
+    }
+
+    @Override
+    public float floatValue(final long row, final long col) {
+        return myDelegate.floatValue(Structure2D.index(myRowsCount, row, col));
+    }
+
     @Override
     public N get(final long index) {
         return myDelegate.get(index);
@@ -552,8 +705,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         int result = 1;
         result = prime * result + (int) (myColumnsCount ^ myColumnsCount >>> 32);
         result = prime * result + (myDelegate == null ? 0 : myDelegate.hashCode());
-        result = prime * result + (int) (myRowsCount ^ myRowsCount >>> 32);
-        return result;
+        return prime * result + (int) (myRowsCount ^ myRowsCount >>> 32);
     }
 
     @Override
@@ -561,6 +713,46 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return myDelegate.indexOfLargest();
     }
 
+    @Override
+    public int intValue(final int index) {
+        return myDelegate.intValue(index);
+    }
+
+    @Override
+    public int intValue(final int row, final int col) {
+        return myDelegate.intValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public int intValue(final long index) {
+        return myDelegate.intValue(index);
+    }
+
+    @Override
+    public int intValue(final long row, final long col) {
+        return myDelegate.intValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public long longValue(final int index) {
+        return myDelegate.longValue(index);
+    }
+
+    @Override
+    public long longValue(final int row, final int col) {
+        return myDelegate.longValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public long longValue(final long index) {
+        return myDelegate.longValue(index);
+    }
+
+    @Override
+    public long longValue(final long row, final long col) {
+        return myDelegate.longValue(Structure2D.index(myRowsCount, row, col));
+    }
+
     @Override
     public double mix(final long row, final long col, final BinaryFunction<N> mixer, final double addend) {
         ProgrammingError.throwIfNull(mixer);
@@ -648,10 +840,12 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return retVal;
     }
 
+    @Override
     public void reset() {
         myDelegate.reset();
     }
 
+    @Override
     public Array2D<N> reshape(final long rows, final long columns) {
         if (Structure2D.count(rows, columns) != this.count()) {
             throw new IllegalArgumentException();
@@ -659,6 +853,71 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return myDelegate.wrapInArray2D(rows);
     }
 
+    @Override
+    public void set(final int index, final byte value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final double value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final float value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final int value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final byte value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final double value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final float value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final int value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final long value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final short value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final int index, final long value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final short value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long index, final byte value) {
+        myDelegate.set(index, value);
+    }
+
     @Override
     public void set(final long index, final Comparable<?> value) {
         myDelegate.set(index, value);
@@ -674,6 +933,21 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         myDelegate.set(index, value);
     }
 
+    @Override
+    public void set(final long index, final int value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long index, final long value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long row, final long col, final byte value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
     @Override
     public void set(final long row, final long col, final Comparable<?> value) {
         myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
@@ -689,6 +963,46 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
     }
 
+    @Override
+    public void set(final long row, final long col, final int value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final long row, final long col, final long value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final long row, final long col, final short value) {
+        myDelegate.set(Structure2D.index(myRowsCount, row, col), value);
+    }
+
+    @Override
+    public void set(final long index, final short value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public short shortValue(final int index) {
+        return myDelegate.shortValue(index);
+    }
+
+    @Override
+    public short shortValue(final int row, final int col) {
+        return myDelegate.shortValue(Structure2D.index(myRowsCount, row, col));
+    }
+
+    @Override
+    public short shortValue(final long index) {
+        return myDelegate.shortValue(index);
+    }
+
+    @Override
+    public short shortValue(final long row, final long col) {
+        return myDelegate.shortValue(Structure2D.index(myRowsCount, row, col));
+    }
+
     @Override
     public Array1D<N> sliceColumn(final long col) {
         return this.sliceColumn(0L, col);
@@ -721,6 +1035,7 @@ public final class Array2D<N extends Comparable<N>> implements Access2D.Visitabl
         return new Array1D<>(myDelegate, Structure2D.index(myRowsCount, row, col), Structure2D.index(myRowsCount, row, myColumnsCount), myRowsCount);
     }
 
+    @Override
     public void supplyTo(final Mutate2D receiver) {
         myDelegate.supplyTo(receiver);
     }
