@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@ import org.ojalgo.function.VoidFunction;
 import org.ojalgo.function.aggregator.Aggregator;
 import org.ojalgo.function.aggregator.AggregatorFunction;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.scalar.Scalar;
@@ -44,6 +45,7 @@ import org.ojalgo.structure.MutateAnyD;
 import org.ojalgo.structure.StructureAnyD;
 import org.ojalgo.structure.TransformationAnyD;
 import org.ojalgo.tensor.TensorFactoryAnyD;
+import org.ojalgo.type.math.MathType;
 
 /**
  * ArrayAnyD
@@ -64,6 +66,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
             myDelegate = new BasicArray.Factory<>(denseArray);
         }
 
+        @Override
         public ArrayAnyD<N> copy(final AccessAnyD<?> source) {
             return myDelegate.copy(source).wrapInArrayAnyD(source.shape());
         }
@@ -73,15 +76,22 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
             return myDelegate.function();
         }
 
+        @Override
+        public MathType getMathType() {
+            return myDelegate.getMathType();
+        }
+
         @Override
         public ArrayAnyD<N> make(final long... structure) {
             return this.makeDense(structure);
         }
 
+        @Override
         public ArrayAnyD<N> makeDense(final long... structure) {
             return myDelegate.makeToBeFilled(structure).wrapInArrayAnyD(structure);
         }
 
+        @Override
         public ArrayAnyD<N> makeFilled(final long[] structure, final NullaryFunction<?> supplier) {
 
             BasicArray<N> toBeFilled = myDelegate.makeToBeFilled(structure);
@@ -91,6 +101,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
             return toBeFilled.wrapInArrayAnyD(structure);
         }
 
+        @Override
         public ArrayAnyD<N> makeSparse(final long... structure) {
             return myDelegate.makeStructuredZero(structure).wrapInArrayAnyD(structure);
         }
@@ -106,14 +117,57 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
 
     }
 
-    public static final Factory<BigDecimal> BIG = new Factory<>(BigArray.FACTORY);
-    public static final Factory<ComplexNumber> COMPLEX = new Factory<>(ComplexArray.FACTORY);
-    public static final Factory<Double> DIRECT32 = new Factory<>(BufferArray.DIRECT32);
-    public static final Factory<Double> DIRECT64 = new Factory<>(BufferArray.DIRECT64);
-    public static final Factory<Double> PRIMITIVE32 = new Factory<>(Primitive32Array.FACTORY);
-    public static final Factory<Double> PRIMITIVE64 = new Factory<>(Primitive64Array.FACTORY);
-    public static final Factory<Quaternion> QUATERNION = new Factory<>(QuaternionArray.FACTORY);
-    public static final Factory<RationalNumber> RATIONAL = new Factory<>(RationalArray.FACTORY);
+    public static final Factory<Double> R032 = ArrayAnyD.factory(ArrayR032.FACTORY);
+    public static final Factory<Double> R064 = ArrayAnyD.factory(ArrayR064.FACTORY);
+    public static final Factory<Quadruple> R128 = ArrayAnyD.factory(ArrayR128.FACTORY);
+    public static final Factory<BigDecimal> R256 = ArrayAnyD.factory(ArrayR256.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #R256} instead
+     */
+    @Deprecated
+    public static final Factory<BigDecimal> BIG = R256;
+    public static final Factory<ComplexNumber> C128 = ArrayAnyD.factory(ArrayC128.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #C128} instead
+     */
+    @Deprecated
+    public static final Factory<ComplexNumber> COMPLEX = C128;
+    /**
+     * @deprecated v52 Use {@link #factory(DenseArray.Factory)} instead
+     */
+    @Deprecated
+    public static final Factory<Double> DIRECT32 = ArrayAnyD.factory(BufferArray.DIRECT32);
+    /**
+     * @deprecated v52 Use {@link #factory(DenseArray.Factory)} instead
+     */
+    @Deprecated
+    public static final Factory<Double> DIRECT64 = ArrayAnyD.factory(BufferArray.DIRECT64);
+    public static final Factory<Quaternion> H256 = ArrayAnyD.factory(ArrayH256.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #R032} instead
+     */
+    @Deprecated
+    public static final Factory<Double> PRIMITIVE32 = R032;
+    /**
+     * @deprecated v52 Use {@link #R064} instead
+     */
+    @Deprecated
+    public static final Factory<Double> PRIMITIVE64 = R064;
+    public static final Factory<RationalNumber> Q128 = ArrayAnyD.factory(ArrayQ128.FACTORY);
+    /**
+     * @deprecated v52 Use {@link #H256} instead
+     */
+    @Deprecated
+    public static final Factory<Quaternion> QUATERNION = H256;
+    /**
+     * @deprecated v52 Use {@link #Q128} instead
+     */
+    @Deprecated
+    public static final Factory<RationalNumber> RATIONAL = Q128;
+    public static final Factory<Double> Z008 = ArrayAnyD.factory(ArrayZ008.FACTORY);
+    public static final Factory<Double> Z016 = ArrayAnyD.factory(ArrayZ016.FACTORY);
+    public static final Factory<Double> Z032 = ArrayAnyD.factory(ArrayZ032.FACTORY);
+    public static final Factory<Double> Z064 = ArrayAnyD.factory(ArrayZ064.FACTORY);
 
     public static <N extends Comparable<N>> ArrayAnyD.Factory<N> factory(final DenseArray.Factory<N> denseArray) {
         return new ArrayAnyD.Factory<>(denseArray);
@@ -135,6 +189,11 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         myStructure = structure;
     }
 
+    @Override
+    public void add(final long index, final byte addend) {
+        myDelegate.add(index, addend);
+    }
+
     @Override
     public void add(final long index, final Comparable<?> addend) {
         myDelegate.add(index, addend);
@@ -150,6 +209,26 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         myDelegate.add(index, addend);
     }
 
+    @Override
+    public void add(final long index, final int addend) {
+        myDelegate.add(index, addend);
+    }
+
+    @Override
+    public void add(final long index, final long addend) {
+        myDelegate.add(index, addend);
+    }
+
+    @Override
+    public void add(final long index, final short addend) {
+        myDelegate.add(index, addend);
+    }
+
+    @Override
+    public void add(final long[] reference, final byte addend) {
+        myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
+    }
+
     @Override
     public void add(final long[] reference, final Comparable<?> addend) {
         myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
@@ -165,6 +244,21 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
     }
 
+    @Override
+    public void add(final long[] reference, final int addend) {
+        myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
+    }
+
+    @Override
+    public void add(final long[] reference, final long addend) {
+        myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
+    }
+
+    @Override
+    public void add(final long[] reference, final short addend) {
+        myDelegate.add(StructureAnyD.index(myStructure, reference), addend);
+    }
+
     @Override
     public N aggregateRange(final long first, final long limit, final Aggregator aggregator) {
         AggregatorFunction<N> visitor = aggregator.getFunction(myDelegate.factory().aggregator());
@@ -186,6 +280,21 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return visitor.get();
     }
 
+    @Override
+    public byte byteValue(final int index) {
+        return myDelegate.byteValue(index);
+    }
+
+    @Override
+    public byte byteValue(final long index) {
+        return myDelegate.byteValue(index);
+    }
+
+    @Override
+    public byte byteValue(final long... ref) {
+        return myDelegate.byteValue(StructureAnyD.index(myStructure, ref));
+    }
+
     @Override
     public long count() {
         return myDelegate.count();
@@ -196,13 +305,18 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return StructureAnyD.count(myStructure, dimension);
     }
 
+    @Override
+    public double doubleValue(final int index) {
+        return myDelegate.doubleValue(index);
+    }
+
     @Override
     public double doubleValue(final long index) {
         return myDelegate.doubleValue(index);
     }
 
     @Override
-    public double doubleValue(final long[] ref) {
+    public double doubleValue(final long... ref) {
         return myDelegate.doubleValue(StructureAnyD.index(myStructure, ref));
     }
 
@@ -228,6 +342,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return true;
     }
 
+    @Override
     public ArrayAnyD<N> expand(final int rank) {
 
         int r = Math.max(this.rank(), rank);
@@ -311,17 +426,33 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
      *
      * @see org.ojalgo.structure.StructureAnyD.Reshapable#flatten()
      */
+    @Override
     public Array1D<N> flatten() {
         return myDelegate.wrapInArray1D();
     }
 
+    @Override
+    public float floatValue(final int index) {
+        return myDelegate.floatValue(index);
+    }
+
+    @Override
+    public float floatValue(final long index) {
+        return myDelegate.floatValue(index);
+    }
+
+    @Override
+    public float floatValue(final long... ref) {
+        return myDelegate.floatValue(StructureAnyD.index(myStructure, ref));
+    }
+
     @Override
     public N get(final long index) {
         return myDelegate.get(index);
     }
 
     @Override
-    public N get(final long[] ref) {
+    public N get(final long... ref) {
         return myDelegate.get(StructureAnyD.index(myStructure, ref));
     }
 
@@ -330,8 +461,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         int prime = 31;
         int result = 1;
         result = prime * result + (myDelegate == null ? 0 : myDelegate.hashCode());
-        result = prime * result + Arrays.hashCode(myStructure);
-        return result;
+        return prime * result + Arrays.hashCode(myStructure);
     }
 
     @Override
@@ -339,6 +469,36 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return myDelegate.indexOfLargest();
     }
 
+    @Override
+    public int intValue(final int index) {
+        return myDelegate.intValue(index);
+    }
+
+    @Override
+    public int intValue(final long index) {
+        return myDelegate.intValue(index);
+    }
+
+    @Override
+    public int intValue(final long... ref) {
+        return myDelegate.intValue(StructureAnyD.index(myStructure, ref));
+    }
+
+    @Override
+    public long longValue(final int index) {
+        return myDelegate.longValue(index);
+    }
+
+    @Override
+    public long longValue(final long index) {
+        return myDelegate.longValue(index);
+    }
+
+    @Override
+    public long longValue(final long... ref) {
+        return myDelegate.longValue(StructureAnyD.index(myStructure, ref));
+    }
+
     @Override
     public double mix(final long[] reference, final BinaryFunction<N> mixer, final double addend) {
         ProgrammingError.throwIfNull(mixer);
@@ -420,31 +580,31 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
     }
 
     @Override
-    public Array2D<N> reduce(final int rowDimension, final int columnDimension, final Aggregator aggregator) {
+    public Array2D<N> reduce(final int rowDim, final int colDim, final Aggregator aggregator) {
 
         long[] structure = this.shape();
 
-        long numberOfRows = structure[rowDimension];
-        long numberOfColumns = structure[columnDimension];
+        long nbRows = structure[rowDim];
+        long nbCols = structure[colDim];
 
         AggregatorFunction<N> visitor = aggregator.getFunction(myDelegate.factory().aggregator());
 
         boolean primitive = myDelegate.isPrimitive();
 
-        Array2D<N> retVal = myDelegate.factory().make(numberOfRows * numberOfColumns).wrapInArray2D(numberOfRows);
+        Array2D<N> retVal = myDelegate.factory().make(nbRows * nbCols).wrapInArray2D(nbRows);
 
-        for (long j = 0L; j < numberOfColumns; j++) {
-            long colInd = j;
+        for (long j = 0L; j < nbCols; j++) {
+            final long col = j;
 
-            for (long i = 0L; i < numberOfRows; i++) {
-                long rowInd = i;
+            for (long i = 0L; i < nbRows; i++) {
+                final long row = i;
 
                 visitor.reset();
-                this.loop(reference -> reference[rowDimension] == rowInd && reference[columnDimension] == colInd, index -> this.visitOne(index, visitor));
+                this.loopReferences(reference -> reference[rowDim] == row && reference[colDim] == col, reference -> this.visitOne(reference, visitor));
                 if (primitive) {
-                    retVal.set(rowInd, colInd, visitor.doubleValue());
+                    retVal.set(row, col, visitor.doubleValue());
                 } else {
-                    retVal.set(rowInd, colInd, visitor.get());
+                    retVal.set(row, col, visitor.get());
                 }
             }
         }
@@ -452,10 +612,12 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return retVal;
     }
 
+    @Override
     public void reset() {
         myDelegate.reset();
     }
 
+    @Override
     public ArrayAnyD<N> reshape(final long... shape) {
         if (StructureAnyD.count(shape) != this.count()) {
             throw new IllegalArgumentException();
@@ -463,6 +625,41 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return myDelegate.wrapInArrayAnyD(shape);
     }
 
+    @Override
+    public void set(final int index, final byte value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final double value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final float value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final int value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final long value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final int index, final short value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long index, final byte value) {
+        myDelegate.set(index, value);
+    }
+
     @Override
     public void set(final long index, final Comparable<?> value) {
         myDelegate.set(index, value);
@@ -478,6 +675,26 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         myDelegate.set(index, value);
     }
 
+    @Override
+    public void set(final long index, final int value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long index, final long value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long index, final short value) {
+        myDelegate.set(index, value);
+    }
+
+    @Override
+    public void set(final long[] reference, final byte value) {
+        myDelegate.set(StructureAnyD.index(myStructure, reference), value);
+    }
+
     @Override
     public void set(final long[] reference, final Comparable<?> value) {
         myDelegate.set(StructureAnyD.index(myStructure, reference), value);
@@ -493,11 +710,41 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         myDelegate.set(StructureAnyD.index(myStructure, reference), value);
     }
 
+    @Override
+    public void set(final long[] reference, final int value) {
+        myDelegate.set(StructureAnyD.index(myStructure, reference), value);
+    }
+
+    @Override
+    public void set(final long[] reference, final long value) {
+        myDelegate.set(StructureAnyD.index(myStructure, reference), value);
+    }
+
+    @Override
+    public void set(final long[] reference, final short value) {
+        myDelegate.set(StructureAnyD.index(myStructure, reference), value);
+    }
+
     @Override
     public long[] shape() {
         return myStructure;
     }
 
+    @Override
+    public short shortValue(final int index) {
+        return myDelegate.shortValue(index);
+    }
+
+    @Override
+    public short shortValue(final long index) {
+        return myDelegate.shortValue(index);
+    }
+
+    @Override
+    public short shortValue(final long... ref) {
+        return myDelegate.shortValue(StructureAnyD.index(myStructure, ref));
+    }
+
     @Override
     public Array1D<N> sliceRange(final long first, final long limit) {
         return myDelegate.wrapInArray1D().sliceRange(first, limit);
@@ -519,6 +766,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return new Array1D<>(myDelegate, first.longValue(), limit.longValue(), step.longValue());
     }
 
+    @Override
     public ArrayAnyD<N> squeeze() {
 
         long[] oldShape = this.shape();
@@ -545,6 +793,7 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
         return this.reshape(shape);
     }
 
+    @Override
     public void supplyTo(final MutateAnyD receiver) {
         myDelegate.supplyTo(receiver);
     }
@@ -604,4 +853,5 @@ public final class ArrayAnyD<N extends Comparable<N>> implements AccessAnyD.Visi
     BasicArray<N> getDelegate() {
         return myDelegate;
     }
+
 }
