@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -131,6 +131,11 @@ final class SegmentedArray<N extends Comparable<N>> extends BasicArray<N> {
         return mySegments[(int) (index >> myIndexBits)].doubleValue(index & myIndexMask);
     }
 
+    @Override
+    public double doubleValue(final int index) {
+        return mySegments[index >> myIndexBits].doubleValue(index & myIndexMask);
+    }
+
     @Override
     public void fillAll(final N value) {
         for (BasicArray<N> tmpSegment : mySegments) {
@@ -465,4 +470,9 @@ final class SegmentedArray<N extends Comparable<N>> extends BasicArray<N> {
         return new SegmentedArray<>(tmpSegments, mySegmentFactory);
     }
 
+    @Override
+    public void set(final int index, final double value) {
+        mySegments[index >> myIndexBits].set(index & myIndexMask, value);
+    }
+
 }
