@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@ import java.util.function.Consumer;
 import java.util.stream.LongStream;
 
 import org.ojalgo.function.BinaryFunction;
-import org.ojalgo.function.FunctionSet;
 import org.ojalgo.function.NullaryFunction;
 import org.ojalgo.function.UnaryFunction;
 import org.ojalgo.function.VoidFunction;
@@ -212,21 +211,18 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
             super(denseFactory);
         }
 
-        public FunctionSet<N> function() {
-            return this.getStrategy().function();
+        /**
+         * @deprecated v53 This will create an array of maximum/infinite length. That's probably not what you
+         *             want. Use {@link #make(long)} instead.
+         */
+        @Deprecated
+        public SparseArray<N> make() {
+            return this.make(Long.MAX_VALUE);
         }
 
         @Override
-        public SparseArray<N> make() {
-            return new SparseArray<>(this.getStrategy());
-        }
-
         public SparseArray<N> make(final long count) {
-            return this.limit(count).make();
-        }
-
-        public Scalar.Factory<N> scalar() {
-            return this.getStrategy().scalar();
+            return new SparseArray<>(this.getDenseFactory(), this.getGrowthStrategy(), count);
         }
 
     }
@@ -243,24 +239,26 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
     private int myActualLength = 0;
     private final long myCount;
     private long[] myIndices;
-    private final DenseCapacityStrategy<N> myStrategy;
+    private final DenseArray.Factory<N> myDenseFactory;
+    private final GrowthStrategy myGrowthStrategy;
     private DenseArray<N> myValues;
     private final N myZeroNumber;
     private final Scalar<N> myZeroScalar;
     private final double myZeroValue;
 
-    SparseArray(final DenseCapacityStrategy<N> strategy) {
+    SparseArray(final DenseArray.Factory<N> denseFactory, final GrowthStrategy growthStrategy, final long count) {
 
-        super(strategy.getDenseFactory());
+        super(denseFactory);
 
-        myCount = strategy.limit();
+        myCount = count;
 
-        myStrategy = strategy;
+        myDenseFactory = denseFactory;
+        myGrowthStrategy = growthStrategy;
 
-        myIndices = new long[strategy.initial()];
-        myValues = strategy.makeInitial();
+        myIndices = new long[growthStrategy.initial()];
+        myValues = growthStrategy.makeInitial(denseFactory);
 
-        myZeroScalar = strategy.scalar().zero();
+        myZeroScalar = denseFactory.scalar().zero();
         myZeroNumber = myZeroScalar.get();
         myZeroValue = myZeroScalar.doubleValue();
     }
@@ -337,6 +335,16 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
         return myZeroValue;
     }
 
+    @Override
+    public double doubleValue(final int index) {
+
+        final int tmpIndex = this.index(index);
+        if (tmpIndex >= 0) {
+            return this.doubleValueInternally(tmpIndex);
+        }
+        return myZeroValue;
+    }
+
     @Override
     public void fillAll(final N value) {
 
@@ -352,7 +360,7 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
 
             if (tmpSize != myIndices.length) {
                 myIndices = Structure1D.newIncreasingRange(0L, tmpSize);
-                myValues = myStrategy.make(tmpSize);
+                myValues = myDenseFactory.make(tmpSize);
                 myActualLength = tmpSize;
             }
 
@@ -369,7 +377,7 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
 
         if (tmpSize != myIndices.length) {
             myIndices = Structure1D.newIncreasingRange(0L, tmpSize);
-            myValues = myStrategy.make(tmpSize);
+            myValues = myDenseFactory.make(tmpSize);
             myActualLength = tmpSize;
         }
 
@@ -595,9 +603,9 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
             } else {
                 // Needs to grow the backing arrays
 
-                final int tmpCapacity = myStrategy.grow(myIndices.length);
+                final int tmpCapacity = myGrowthStrategy.grow(myIndices.length);
                 final long[] tmpIndices = new long[tmpCapacity];
-                final DenseArray<N> tmpValues = myStrategy.make(tmpCapacity);
+                final DenseArray<N> tmpValues = myDenseFactory.make(tmpCapacity);
 
                 for (int i = 0; i < tmpInsInd; i++) {
                     tmpIndices[i] = myIndices[i];
@@ -647,9 +655,9 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
             } else {
                 // Needs to grow the backing arrays
 
-                final int tmpCapacity = myStrategy.grow(myIndices.length);
+                final int tmpCapacity = myGrowthStrategy.grow(myIndices.length);
                 final long[] tmpIndices = new long[tmpCapacity];
-                final DenseArray<N> tmpValues = myStrategy.make(tmpCapacity);
+                final DenseArray<N> tmpValues = myDenseFactory.make(tmpCapacity);
 
                 for (int i = 0; i < tmpInsInd; i++) {
                     tmpIndices[i] = myIndices[i];
@@ -734,7 +742,7 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
 
         for (int i = 0; i < myIndices.length; i++) {
             final long tmpIndex = myIndices[i];
-            if ((tmpIndex >= first && tmpIndex < limit) && ((tmpIndex - first) % step == 0L)) {
+            if (tmpIndex >= first && tmpIndex < limit && (tmpIndex - first) % step == 0L) {
                 tmpValue = PrimitiveMath.ABS.invoke(myValues.doubleValue(i));
                 if (tmpValue > tmpLargest) {
                     tmpLargest = tmpValue;
@@ -757,7 +765,7 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
         }
         for (int i = 0; i < myIndices.length; i++) {
             final long tmpIndex = myIndices[i];
-            if ((tmpIndex >= first && tmpIndex < limit) && ((tmpIndex - first) % step == 0L)) {
+            if (tmpIndex >= first && tmpIndex < limit && (tmpIndex - first) % step == 0L) {
                 myValues.modify(tmpIndex, i, left, function);
             }
         }
@@ -817,7 +825,7 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
 
     DenseArray<N> densify() {
 
-        final DenseArray<N> retVal = myStrategy.make((int) this.count());
+        final DenseArray<N> retVal = myDenseFactory.make((int) this.count());
 
         if (this.isPrimitive()) {
             for (int i = 0; i < myActualLength; i++) {
@@ -873,6 +881,11 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
                 return limit - first;
             }
 
+            @Override
+            public double doubleValue(final int index) {
+                return myValues.doubleValue(first + index);
+            }
+
             @Override
             public double doubleValue(final long index) {
                 return myValues.doubleValue(first + index);
@@ -928,4 +941,12 @@ public final class SparseArray<N extends Comparable<N>> extends BasicArray<N> {
         }
     }
 
+    @Override
+    public void set(final int index, final double value) {
+
+        int internalIndex = this.index(index);
+
+        this.update(index, internalIndex, value, false);
+    }
+
 }
