@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@ import java.util.List;
 import org.ojalgo.array.ArrayR064;
 import org.ojalgo.array.BasicArray;
 import org.ojalgo.array.DenseArray;
-import org.ojalgo.array.Primitive64Array;
 import org.ojalgo.array.SparseArray;
 import org.ojalgo.function.UnaryFunction;
 import org.ojalgo.structure.Access1D;
@@ -39,18 +38,18 @@ import org.ojalgo.type.NumberDefinition;
 public final class Equation implements Comparable<Equation>, Access1D<Double>, Mutate1D.Modifiable<Double> {
 
     public static Equation dense(final int pivot, final int cols) {
-        return Equation.dense(pivot, cols, Primitive64Array.FACTORY);
+        return Equation.dense(pivot, cols, ArrayR064.FACTORY);
     }
 
-    public static Equation dense(final int pivot, final int cols, final DenseArray.Factory<Double> factory) {
+    public static <N extends Comparable<N>> Equation dense(final int pivot, final int cols, final DenseArray.Factory<N> factory) {
         return new Equation(factory.make(cols), pivot, ZERO);
     }
 
     public static List<Equation> denseSystem(final int rows, final int cols) {
-        return Equation.denseSystem(rows, cols, Primitive64Array.FACTORY);
+        return Equation.denseSystem(rows, cols, ArrayR064.FACTORY);
     }
 
-    public static List<Equation> denseSystem(final int rows, final int cols, final DenseArray.Factory<Double> factory) {
+    public static <N extends Comparable<N>> List<Equation> denseSystem(final int rows, final int cols, final DenseArray.Factory<N> factory) {
 
         List<Equation> system = new ArrayList<>(rows);
 
@@ -66,49 +65,54 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
     }
 
     public static Equation sparse(final int pivot, final int cols) {
-        return Equation.sparse(pivot, cols, Primitive64Array.FACTORY);
+        return Equation.sparse(pivot, cols, ArrayR064.FACTORY);
     }
 
-    public static Equation sparse(final int pivot, final int cols, final DenseArray.Factory<Double> factory) {
-        return new Equation(SparseArray.factory(factory).limit(cols).make(), pivot, ZERO);
+    public static <N extends Comparable<N>> Equation sparse(final int pivot, final int cols, final DenseArray.Factory<N> factory) {
+        return new Equation(SparseArray.factory(factory).make(cols), pivot, ZERO);
     }
 
-    public static Equation sparse(final int pivot, final int cols, final DenseArray.Factory<Double> factory, final int numberOfNonzeros) {
-        return new Equation(SparseArray.factory(factory).limit(cols).initial(numberOfNonzeros).make(), pivot, ZERO);
+    public static <N extends Comparable<N>> Equation sparse(final int pivot, final int cols, final DenseArray.Factory<N> factory, final int numberOfNonzeros) {
+        return new Equation(SparseArray.factory(factory).initial(numberOfNonzeros).make(cols), pivot, ZERO);
     }
 
     public static Equation sparse(final int pivot, final int cols, final int numberOfNonzeros) {
-        return Equation.sparse(pivot, cols, Primitive64Array.FACTORY, numberOfNonzeros);
+        return Equation.sparse(pivot, cols, ArrayR064.FACTORY, numberOfNonzeros);
     }
 
     public static List<Equation> sparseSystem(final int rows, final int cols) {
-        return Equation.sparseSystem(rows, cols, Primitive64Array.FACTORY);
+        return Equation.sparseSystem(rows, cols, ArrayR064.FACTORY);
     }
 
-    public static List<Equation> sparseSystem(final int rows, final int cols, final DenseArray.Factory<Double> factory) {
+    public static <N extends Comparable<N>> List<Equation> sparseSystem(final int rows, final int cols, final DenseArray.Factory<N> factory) {
 
         List<Equation> system = new ArrayList<>(rows);
 
         for (int i = 0; i < rows; i++) {
-            system.add(new Equation(SparseArray.factory(factory).limit(cols).make(), i, ZERO));
+            system.add(new Equation(SparseArray.factory(factory).make(cols), i, ZERO));
         }
 
         return system;
     }
 
-    public static List<Equation> sparseSystem(final int rows, final int cols, final DenseArray.Factory<Double> factory, final int numberOfNonzeros) {
+    public static <N extends Comparable<N>> List<Equation> sparseSystem(final int rows, final int cols, final DenseArray.Factory<N> factory,
+            final int numberOfNonzeros) {
 
         List<Equation> system = new ArrayList<>(rows);
 
         for (int i = 0; i < rows; i++) {
-            system.add(new Equation(SparseArray.factory(factory).limit(cols).initial(numberOfNonzeros).make(), i, ZERO));
+            system.add(new Equation(SparseArray.factory(factory).initial(numberOfNonzeros).make(cols), i, ZERO));
         }
 
         return system;
     }
 
     public static List<Equation> sparseSystem(final int rows, final int cols, final int numberOfNonzeros) {
-        return Equation.sparseSystem(rows, cols, Primitive64Array.FACTORY, numberOfNonzeros);
+        return Equation.sparseSystem(rows, cols, ArrayR064.FACTORY, numberOfNonzeros);
+    }
+
+    public static Equation wrap(final BasicArray<?> body, final int pivot, final double rhs) {
+        return new Equation(body, pivot, rhs);
     }
 
     /**
@@ -118,7 +122,7 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
     /**
      * The (nonzero) elements of this equation/row
      */
-    private final BasicArray<Double> myElements;
+    private final BasicArray<?> myBody;
     private double myPivot = ZERO;
     private double myRHS;
 
@@ -127,36 +131,30 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
      */
     @Deprecated
     public Equation(final int row, final long numberOfColumns, final double rhs) {
-        this(SparseArray.factory(ArrayR064.FACTORY).limit(numberOfColumns).make(), row, rhs);
+        this(SparseArray.factory(ArrayR064.FACTORY).make(numberOfColumns), row, rhs);
     }
 
-    /**
-     * @deprecated v49 Use one of the factory methods instead
-     */
-    @Deprecated
-    public Equation(final int row, final long numberOfColumns, final double rhs, final int numberOfNonzeros) {
-        this(SparseArray.factory(ArrayR064.FACTORY).limit(numberOfColumns).initial(numberOfNonzeros).make(), row, rhs);
-    }
-
-    Equation(final BasicArray<Double> elements, final int pivot, final double rhs) {
+    Equation(final BasicArray<?> body, final int pivot, final double rhs) {
 
         super();
 
-        myElements = elements;
+        myBody = body;
         myRHS = rhs;
 
         index = pivot;
-        myPivot = ZERO;
+        myPivot = myBody.doubleValue(pivot);
     }
 
+    @Override
     public void add(final long ind, final Comparable<?> addend) {
         this.add(ind, NumberDefinition.doubleValue(addend));
     }
 
+    @Override
     public void add(final long ind, final double addend) {
-        myElements.add(ind, addend);
+        myBody.add(ind, addend);
         if (ind == index) {
-            myPivot = myElements.doubleValue(ind);
+            myPivot = myBody.doubleValue(ind);
         }
     }
 
@@ -171,20 +169,24 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
         return this.calculate(x, myRHS, relaxation);
     }
 
+    @Override
     public int compareTo(final Equation other) {
         return Integer.compare(index, other.index);
     }
 
+    @Override
     public long count() {
-        return myElements.count();
+        return myBody.count();
     }
 
+    @Override
     public double dot(final Access1D<?> vector) {
-        return myElements.dot(vector);
+        return myBody.dot(vector);
     }
 
-    public double doubleValue(final long ind) {
-        return myElements.doubleValue(ind);
+    @Override
+    public double doubleValue(final int ind) {
+        return myBody.doubleValue(ind);
     }
 
     @Override
@@ -202,8 +204,13 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
         return true;
     }
 
+    @Override
     public Double get(final long ind) {
-        return myElements.get(ind);
+        return Double.valueOf(myBody.doubleValue(ind));
+    }
+
+    public BasicArray<?> getBody() {
+        return myBody;
     }
 
     /**
@@ -231,10 +238,13 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
         this.calculate(x, ZERO, ONE);
     }
 
+    @Override
     public void modifyOne(final long ind, final UnaryFunction<Double> modifier) {
-        myElements.modifyOne(ind, modifier);
+        double value = myBody.doubleValue(ind);
+        value = modifier.invoke(value);
+        myBody.set(ind, value);
         if (ind == index) {
-            myPivot = myElements.doubleValue(ind);
+            myPivot = value;
         }
     }
 
@@ -243,7 +253,7 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
     }
 
     public void set(final long ind, final double value) {
-        myElements.set(ind, value);
+        myBody.set(ind, value);
         if (ind == index) {
             myPivot = value;
         }
@@ -255,14 +265,14 @@ public final class Equation implements Comparable<Equation>, Access1D<Double>, M
 
     @Override
     public String toString() {
-        return index + ": " + myElements.toString() + " = " + myRHS;
+        return index + ": " + myBody.toString() + " = " + myRHS;
     }
 
     private <T extends Access1D<Double> & Mutate1D.Modifiable<Double>> double calculate(final T x, final double rhs, final double relaxation) {
 
         double increment = rhs;
 
-        double error = increment -= myElements.dot(x);
+        double error = increment -= myBody.dot(x);
 
         increment *= relaxation;
 
