@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,6 +44,7 @@ import org.ojalgo.structure.Mutate2D;
 import org.ojalgo.structure.Structure2D;
 import org.ojalgo.tensor.TensorFactory1D;
 import org.ojalgo.tensor.TensorFactory2D;
+import org.ojalgo.type.math.MathType;
 
 /**
  * MatrixFactory creates instances of classes that implement the {@linkplain org.ojalgo.matrix.BasicMatrix}
@@ -199,6 +200,10 @@ public abstract class MatrixFactory<N extends Comparable<N>, M extends BasicMatr
                 return MatrixFactory.this.scalar();
             }
 
+            public MathType getMathType() {
+                return MatrixFactory.this.getMathType();
+            }
+
         });
     }
 
@@ -217,6 +222,10 @@ public abstract class MatrixFactory<N extends Comparable<N>, M extends BasicMatr
                 return MatrixFactory.this.scalar();
             }
 
+            public MathType getMathType() {
+                return MatrixFactory.this.getMathType();
+            }
+
         });
     }
 
@@ -239,4 +248,8 @@ public abstract class MatrixFactory<N extends Comparable<N>, M extends BasicMatr
 
     abstract SR sparse(final SparseStore<N> store);
 
+    public MathType getMathType() {
+        return myPhysicalFactory.getMathType();
+    }
+
 }
