@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -83,6 +83,10 @@ final class RawSingularValue extends RawDecomposition implements SingularValue<D
         super();
     }
 
+    public void btran(final PhysicalStore<Double> arg) {
+        arg.fillByMultiplying(this.getInverse().transpose(), arg.copy());
+    }
+
     public boolean computeValuesOnly(final Access2D.Collectable<Double, ? super PhysicalStore<Double>> matrix) {
         return this.doDecompose(matrix, false);
     }
