@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@ import org.ojalgo.array.PlainArray;
 import org.ojalgo.matrix.Provider2D;
 import org.ojalgo.matrix.store.MatrixStore;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.structure.Structure2D;
@@ -59,6 +60,7 @@ public interface SingularValue<N extends Comparable<N>> extends MatrixDecomposit
             return this.make(TYPICAL, fullSize);
         }
 
+        @Override
         default SingularValue<N> make(final Structure2D typical) {
             return this.make(typical, false);
         }
@@ -67,18 +69,51 @@ public interface SingularValue<N extends Comparable<N>> extends MatrixDecomposit
 
     }
 
-    Factory<ComplexNumber> COMPLEX = (typical, fullSize) -> new SingularValueDecomposition.Complex(fullSize);
+    Factory<ComplexNumber> C128 = (typical, fullSize) -> new SingularValueDecomposition.C128(fullSize);
 
-    Factory<Double> PRIMITIVE = (typical, fullSize) -> {
+    /**
+     * @deprecated
+     */
+    @Deprecated
+    Factory<ComplexNumber> COMPLEX = C128;
+
+    Factory<Quaternion> H256 = (typical, fullSize) -> new SingularValueDecomposition.H256(fullSize);
+
+    Factory<Double> R064 = (typical, fullSize) -> {
         if (fullSize || 1024L < typical.countColumns() && typical.count() <= PlainArray.MAX_SIZE) {
-            return new SingularValueDecomposition.Primitive(fullSize);
+            return new SingularValueDecomposition.R064(fullSize);
+        } else {
+            return new RawSingularValue();
         }
-        return new RawSingularValue();
     };
 
-    Factory<Quaternion> QUATERNION = (typical, fullSize) -> new SingularValueDecomposition.Quat(fullSize);
+    /**
+     * @deprecated
+     */
+    @Deprecated
+    Factory<Double> PRIMITIVE = R064;
 
-    Factory<RationalNumber> RATIONAL = (typical, fullSize) -> new SingularValueDecomposition.Rational(fullSize);
+    Factory<RationalNumber> Q128 = (typical, fullSize) -> new SingularValueDecomposition.Q128(fullSize);
+
+    Factory<Quadruple> R128 = (typical, fullSize) -> new SingularValueDecomposition.R128(fullSize);
+
+    /**
+     * @deprecated
+     */
+    @Deprecated
+    Factory<Quadruple> QUADRUPLE = R128;
+
+    /**
+     * @deprecated
+     */
+    @Deprecated
+    Factory<Quaternion> QUATERNION = H256;
+
+    /**
+     * @deprecated
+     */
+    @Deprecated
+    Factory<RationalNumber> RATIONAL = Q128;
 
     static <N extends Comparable<N>> boolean equals(final MatrixStore<N> matrix, final SingularValue<N> decomposition, final NumberContext context) {
 
@@ -149,6 +184,7 @@ public interface SingularValue<N extends Comparable<N>> extends MatrixDecomposit
      *
      * @return The largest singular value divided by the smallest singular value.
      */
+    @Override
     double getCondition();
 
     /**
@@ -229,6 +265,7 @@ public interface SingularValue<N extends Comparable<N>> extends MatrixDecomposit
      */
     MatrixStore<N> getV();
 
+    @Override
     default MatrixStore<N> reconstruct() {
         MatrixStore<N> mtrxQ1 = this.getU();
         MatrixStore<N> mtrxD = this.getD();
@@ -236,4 +273,15 @@ public interface SingularValue<N extends Comparable<N>> extends MatrixDecomposit
         return mtrxQ1.multiply(mtrxD).multiply(mtrxQ2.conjugate());
     }
 
+    default MatrixStore<N> reconstruct(final int k) {
+
+        int limit = Math.min(k, this.getRank());
+
+        MatrixStore<N> mtrxQ1 = this.getU().limits(-1, limit);
+        MatrixStore<N> mtrxD = this.getD().limits(limit, limit);
+        MatrixStore<N> mtrxQ2 = this.getV().limits(-1, limit);
+
+        return mtrxQ1.multiply(mtrxD).multiply(mtrxQ2.conjugate());
+    }
+
 }
