@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,7 @@ import org.ojalgo.matrix.store.PhysicalStore;
 import org.ojalgo.matrix.store.Primitive64Store;
 import org.ojalgo.netio.BasicLogger;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.scalar.Scalar;
@@ -48,50 +49,62 @@ import org.ojalgo.type.context.NumberContext;
 
 abstract class SingularValueDecomposition<N extends Comparable<N>> extends GenericDecomposition<N> implements SingularValue<N> {
 
-    static final class Complex extends SingularValueDecomposition<ComplexNumber> {
+    static final class C128 extends SingularValueDecomposition<ComplexNumber> {
 
-        Complex() {
+        C128() {
             this(false);
         }
 
-        Complex(final boolean fullSize) {
-            super(GenericStore.COMPLEX, new BidiagonalDecomposition.Complex(fullSize), fullSize);
+        C128(final boolean fullSize) {
+            super(GenericStore.C128, new BidiagonalDecomposition.C128(fullSize), fullSize);
         }
 
     }
 
-    static final class Primitive extends SingularValueDecomposition<Double> {
+    static final class H256 extends SingularValueDecomposition<Quaternion> {
 
-        Primitive() {
+        H256() {
             this(false);
         }
 
-        Primitive(final boolean fullSize) {
-            super(Primitive64Store.FACTORY, new BidiagonalDecomposition.Primitive(fullSize), fullSize);
+        H256(final boolean fullSize) {
+            super(GenericStore.H256, new BidiagonalDecomposition.H256(fullSize), fullSize);
         }
 
     }
 
-    static final class Quat extends SingularValueDecomposition<Quaternion> {
+    static final class Q128 extends SingularValueDecomposition<RationalNumber> {
 
-        Quat() {
+        Q128() {
             this(false);
         }
 
-        Quat(final boolean fullSize) {
-            super(GenericStore.QUATERNION, new BidiagonalDecomposition.Quat(fullSize), fullSize);
+        Q128(final boolean fullSize) {
+            super(GenericStore.Q128, new BidiagonalDecomposition.Q128(fullSize), fullSize);
         }
 
     }
 
-    static final class Rational extends SingularValueDecomposition<RationalNumber> {
+    static final class R064 extends SingularValueDecomposition<Double> {
 
-        Rational() {
+        R064() {
             this(false);
         }
 
-        Rational(final boolean fullSize) {
-            super(GenericStore.RATIONAL, new BidiagonalDecomposition.Rational(fullSize), fullSize);
+        R064(final boolean fullSize) {
+            super(Primitive64Store.FACTORY, new BidiagonalDecomposition.R064(fullSize), fullSize);
+        }
+
+    }
+
+    static final class R128 extends SingularValueDecomposition<Quadruple> {
+
+        R128() {
+            this(false);
+        }
+
+        R128(final boolean fullSize) {
+            super(GenericStore.R128, new BidiagonalDecomposition.R128(fullSize), fullSize);
         }
 
     }
@@ -320,10 +333,12 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
     private final boolean myFullSize;
     private final Structure2D myInputStructure = new Structure2D() {
 
+        @Override
         public long countColumns() {
             return myTransposed ? myBidiagonal.getRowDim() : myBidiagonal.getColDim();
         }
 
+        @Override
         public long countRows() {
             return myTransposed ? myBidiagonal.getColDim() : myBidiagonal.getRowDim();
         }
@@ -350,10 +365,17 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         myFullSize = fullSize;
     }
 
+    @Override
+    public final void btran(final PhysicalStore<N> arg) {
+        arg.fillByMultiplying(this.getInverse().transpose(), arg.copy());
+    }
+
+    @Override
     public boolean computeValuesOnly(final Access2D.Collectable<N, ? super PhysicalStore<N>> matrix) {
         return this.compute(matrix, true, false);
     }
 
+    @Override
     public int countSignificant(final double threshold) {
         int significant = 0;
         for (int i = 0; i < s.length; i++) {
@@ -364,6 +386,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return significant;
     }
 
+    @Override
     public boolean decompose(final Access2D.Collectable<N, ? super PhysicalStore<N>> matrix) {
         return this.compute(matrix, false, this.isFullSize());
     }
@@ -373,6 +396,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return myTransposed ? myBidiagonal.getRowDim() : myBidiagonal.getColDim();
     }
 
+    @Override
     public double getCondition() {
 
         final Array1D<Double> tmpSingularValues = this.getSingularValues();
@@ -380,6 +404,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return tmpSingularValues.doubleValue(0) / tmpSingularValues.doubleValue(tmpSingularValues.length - 1);
     }
 
+    @Override
     public MatrixStore<N> getCovariance() {
 
         MatrixStore<N> v = this.getV();
@@ -395,6 +420,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return tmp.multiply(tmp.transpose());
     }
 
+    @Override
     public MatrixStore<N> getD() {
 
         if (this.isComputed() && myD == null) {
@@ -404,6 +430,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return myD;
     }
 
+    @Override
     public double getFrobeniusNorm() {
 
         double retVal = ZERO;
@@ -419,10 +446,12 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return SQRT.invoke(retVal);
     }
 
+    @Override
     public MatrixStore<N> getInverse() {
         return this.getInverse(this.preallocate(myInputStructure));
     }
 
+    @Override
     public MatrixStore<N> getInverse(final PhysicalStore<N> preallocated) {
 
         if (myInverse == null) {
@@ -441,12 +470,14 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
             }
 
             preallocated.fillByMultiplying(tmpMtrx, this.getU().limits(-1, rank).conjugate());
+
             myInverse = preallocated;
         }
 
         return myInverse;
     }
 
+    @Override
     public double getKyFanNorm(final int k) {
 
         final Array1D<Double> tmpSingularValues = this.getSingularValues();
@@ -460,10 +491,12 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return retVal;
     }
 
+    @Override
     public double getOperatorNorm() {
         return this.getSingularValues().doubleValue(0);
     }
 
+    @Override
     public double getRankThreshold() {
         return Math.max(MACHINE_SMALLEST, s[0]) * this.getDimensionalEpsilon();
     }
@@ -473,6 +506,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return myTransposed ? myBidiagonal.getColDim() : myBidiagonal.getRowDim();
     }
 
+    @Override
     public Array1D<Double> getSingularValues() {
 
         if (mySingularValues == null && this.isComputed()) {
@@ -482,19 +516,23 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return mySingularValues;
     }
 
+    @Override
     public MatrixStore<N> getSolution(final Collectable<N, ? super PhysicalStore<N>> rhs) {
         return this.getInverse().multiply(this.collect(rhs));
     }
 
+    @Override
     public MatrixStore<N> getSolution(final Collectable<N, ? super PhysicalStore<N>> rhs, final PhysicalStore<N> preallocated) {
         preallocated.fillByMultiplying(this.getInverse(), this.collect(rhs));
         return preallocated;
     }
 
+    @Override
     public double getTraceNorm() {
         return this.getKyFanNorm(this.getSingularValues().size());
     }
 
+    @Override
     public MatrixStore<N> getU() {
 
         if (!myValuesOnly && this.isComputed() && myU == null) {
@@ -509,6 +547,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return myU;
     }
 
+    @Override
     public MatrixStore<N> getV() {
 
         if (!myValuesOnly && this.isComputed() && myV == null) {
@@ -522,6 +561,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return myV;
     }
 
+    @Override
     public MatrixStore<N> invert(final Access2D<?> original) throws RecoverableCondition {
 
         this.decompose(this.wrap(original));
@@ -532,24 +572,29 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         throw RecoverableCondition.newMatrixNotInvertible();
     }
 
+    @Override
     public MatrixStore<N> invert(final Access2D<?> original, final PhysicalStore<N> preallocated) throws RecoverableCondition {
 
         this.decompose(this.wrap(original));
 
         if (this.isSolvable()) {
             return this.getInverse(preallocated);
+        } else {
+            throw RecoverableCondition.newMatrixNotInvertible();
         }
-        throw RecoverableCondition.newMatrixNotInvertible();
     }
 
+    @Override
     public boolean isFullRank() {
         return s[s.length - 1] > this.getRankThreshold();
     }
 
+    @Override
     public boolean isFullSize() {
         return myFullSize;
     }
 
+    @Override
     public boolean isOrdered() {
         return true;
     }
@@ -559,10 +604,12 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         return super.isSolvable();
     }
 
+    @Override
     public PhysicalStore<N> preallocate(final Structure2D template) {
         return this.allocate(template.countColumns(), template.countRows());
     }
 
+    @Override
     public PhysicalStore<N> preallocate(final Structure2D templateBody, final Structure2D templateRHS) {
         return this.allocate(templateBody.countColumns(), templateRHS.countColumns());
     }
@@ -585,6 +632,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         myTransposed = false;
     }
 
+    @Override
     public MatrixStore<N> solve(final Access2D<?> body, final Access2D<?> rhs) throws RecoverableCondition {
 
         this.decompose(this.wrap(body));
@@ -595,6 +643,7 @@ abstract class SingularValueDecomposition<N extends Comparable<N>> extends Gener
         throw RecoverableCondition.newEquationSystemNotSolvable();
     }
 
+    @Override
     public MatrixStore<N> solve(final Access2D<?> body, final Access2D<?> rhs, final PhysicalStore<N> preallocated) throws RecoverableCondition {
 
         this.decompose(this.wrap(body));
