@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,14 +25,7 @@ import java.util.Arrays;
 import java.util.List;
 
 import org.ojalgo.ProgrammingError;
-import org.ojalgo.array.Array1D;
-import org.ojalgo.array.Array2D;
-import org.ojalgo.array.ArrayC128;
-import org.ojalgo.array.ArrayH256;
-import org.ojalgo.array.ArrayQ128;
-import org.ojalgo.array.BasicArray;
-import org.ojalgo.array.DenseArray;
-import org.ojalgo.array.ScalarArray;
+import org.ojalgo.array.*;
 import org.ojalgo.array.operation.*;
 import org.ojalgo.concurrent.DivideAndConquer;
 import org.ojalgo.function.BinaryFunction;
@@ -53,11 +46,13 @@ import org.ojalgo.matrix.transformation.Householder;
 import org.ojalgo.matrix.transformation.HouseholderReference;
 import org.ojalgo.matrix.transformation.Rotation;
 import org.ojalgo.scalar.ComplexNumber;
+import org.ojalgo.scalar.Quadruple;
 import org.ojalgo.scalar.Quaternion;
 import org.ojalgo.scalar.RationalNumber;
 import org.ojalgo.scalar.Scalar;
 import org.ojalgo.structure.Access1D;
 import org.ojalgo.structure.Access2D;
+import org.ojalgo.type.math.MathType;
 
 /**
  * A generic implementation of {@linkplain PhysicalStore}.
@@ -75,14 +70,17 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             myDenseArrayFactory = denseArrayFactory;
         }
 
+        @Override
         public AggregatorSet<N> aggregator() {
             return myDenseArrayFactory.function().aggregator();
         }
 
+        @Override
         public DenseArray.Factory<N> array() {
             return myDenseArrayFactory;
         }
 
+        @Override
         public GenericStore<N> columns(final Access1D<?>... source) {
 
             int tmpRowDim = source[0].size();
@@ -101,6 +99,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> columns(final Comparable<?>[]... source) {
 
             int tmpRowDim = source[0].length;
@@ -119,6 +118,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> columns(final double[]... source) {
 
             int tmpRowDim = source[0].length;
@@ -137,6 +137,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> columns(final List<? extends Comparable<?>>... source) {
 
             int tmpRowDim = source[0].size();
@@ -155,6 +156,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> conjugate(final Access2D<?> source) {
 
             GenericStore<N> retVal = new GenericStore<>(this, (int) source.countColumns(), (int) source.countRows());
@@ -183,6 +185,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return retVal;
         }
 
+        @Override
         public GenericStore<N> copy(final Access2D<?> source) {
 
             int tmpRowDim = source.getRowDim();
@@ -211,26 +214,37 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return retVal;
         }
 
+        @Override
         public FunctionSet<N> function() {
             return myDenseArrayFactory.function();
         }
 
+        @Override
+        public MathType getMathType() {
+            return myDenseArrayFactory.getMathType();
+        }
+
+        @Override
         public GenericStore<N> make(final long rows, final long columns) {
             return new GenericStore<>(this, (int) rows, (int) columns);
         }
 
+        @Override
         public Householder.Generic<N> makeHouseholder(final int length) {
             return new Householder.Generic<>(myDenseArrayFactory.scalar(), length);
         }
 
+        @Override
         public Rotation.Generic<N> makeRotation(final int low, final int high, final double cos, final double sin) {
             return this.makeRotation(low, high, myDenseArrayFactory.scalar().cast(cos), myDenseArrayFactory.scalar().cast(sin));
         }
 
+        @Override
         public Rotation.Generic<N> makeRotation(final int low, final int high, final N cos, final N sin) {
             return new Rotation.Generic<>(low, high, cos, sin);
         }
 
+        @Override
         public GenericStore<N> rows(final Access1D<?>... source) {
 
             final int tmpRowDim = source.length;
@@ -249,6 +263,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> rows(final Comparable<?>[]... source) {
 
             final int tmpRowDim = source.length;
@@ -267,6 +282,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> rows(final double[]... source) {
 
             final int tmpRowDim = source.length;
@@ -285,6 +301,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public GenericStore<N> rows(final List<? extends Comparable<?>>... source) {
 
             final int tmpRowDim = source.length;
@@ -303,10 +320,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
             return new GenericStore<>(this, tmpRowDim, tmpColDim, tmpData);
         }
 
+        @Override
         public Scalar.Factory<N> scalar() {
             return myDenseArrayFactory.scalar();
         }
 
+        @Override
         public GenericStore<N> transpose(final Access2D<?> source) {
 
             GenericStore<N> retVal = new GenericStore<>(this, (int) source.countColumns(), (int) source.countRows());
@@ -337,9 +356,30 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
 
     }
 
-    public static final PhysicalStore.Factory<ComplexNumber, GenericStore<ComplexNumber>> COMPLEX = new GenericStore.Factory<>(ArrayC128.FACTORY);
-    public static final PhysicalStore.Factory<Quaternion, GenericStore<Quaternion>> QUATERNION = new GenericStore.Factory<>(ArrayH256.FACTORY);
-    public static final PhysicalStore.Factory<RationalNumber, GenericStore<RationalNumber>> RATIONAL = new GenericStore.Factory<>(ArrayQ128.FACTORY);
+    public static final PhysicalStore.Factory<ComplexNumber, GenericStore<ComplexNumber>> C128 = new GenericStore.Factory<>(ArrayC128.FACTORY);
+    /**
+     * @deprecated Use {@link #C128} instead
+     */
+    @Deprecated
+    public static final PhysicalStore.Factory<ComplexNumber, GenericStore<ComplexNumber>> COMPLEX = C128;
+    public static final PhysicalStore.Factory<Quaternion, GenericStore<Quaternion>> H256 = new GenericStore.Factory<>(ArrayH256.FACTORY);
+    public static final PhysicalStore.Factory<RationalNumber, GenericStore<RationalNumber>> Q128 = new GenericStore.Factory<>(ArrayQ128.FACTORY);
+    public static final PhysicalStore.Factory<Quadruple, GenericStore<Quadruple>> R128 = new GenericStore.Factory<>(ArrayR128.FACTORY);
+    /**
+     * @deprecated Use {@link #R128} instead
+     */
+    @Deprecated
+    public static final PhysicalStore.Factory<Quadruple, GenericStore<Quadruple>> QUADRUPLE = R128;
+    /**
+     * @deprecated Use {@link #H256} instead
+     */
+    @Deprecated
+    public static final PhysicalStore.Factory<Quaternion, GenericStore<Quaternion>> QUATERNION = H256;
+    /**
+     * @deprecated Use {@link #Q128} instead
+     */
+    @Deprecated
+    public static final PhysicalStore.Factory<RationalNumber, GenericStore<RationalNumber>> RATIONAL = Q128;
 
     public static <N extends Scalar<N>> GenericStore<N> wrap(final GenericStore.Factory<N> factory, final N... data) {
         return new GenericStore<>(factory, data.length, 1, data);
@@ -403,6 +443,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         multiplyNeither = MultiplyNeither.newGeneric(myRowDim, myColDim);
     }
 
+    @Override
     public void accept(final Access2D<?> supplied) {
         for (long j = 0L; j < supplied.countColumns(); j++) {
             for (long i = 0L; i < supplied.countRows(); i++) {
@@ -411,14 +452,17 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void add(final long row, final long col, final Comparable<?> addend) {
         myUtility.add(row, col, addend);
     }
 
+    @Override
     public void add(final long row, final long col, final double addend) {
         myUtility.add(row, col, addend);
     }
 
+    @Override
     public void applyCholesky(final int iterationPoint, final BasicArray<N> multipliers) {
 
         final N[] tmpData = data;
@@ -442,6 +486,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void applyLDL(final int iterationPoint, final BasicArray<N> multipliers) {
 
         final N[] tmpData = data;
@@ -465,6 +510,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void applyLU(final int iterationPoint, final BasicArray<N> multipliers) {
 
         final N[] tmpData = data;
@@ -488,48 +534,53 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public Array1D<N> asList() {
         return myUtility.flatten();
     }
 
+    @Override
     public Array1D<ComplexNumber> computeInPlaceSchur(final PhysicalStore<N> transformationCollector, final boolean eigenvalue) {
         ProgrammingError.throwForUnsupportedOptionalOperation();
         return null;
     }
 
+    @Override
     public MatrixStore<N> conjugate() {
         return new ConjugatedStore<>(this);
     }
 
+    @Override
     public GenericStore<N> copy() {
         return new GenericStore<>(myFactory, myRowDim, myColDim, this.copyOfData());
     }
 
+    @Override
     public long countColumns() {
         return myColDim;
     }
 
+    @Override
     public long countRows() {
         return myRowDim;
     }
 
+    @Override
     public void divideAndCopyColumn(final int row, final int column, final BasicArray<N> destination) {
 
-        final N[] tmpData = data;
-        final int tmpRowDim = myRowDim;
+        N[] destinationData = ((ScalarArray<N>) destination).data;
 
-        final N[] tmpDestination = ((ScalarArray<N>) destination).data;
+        int index = row + column * myRowDim;
+        N denominator = data[index];
 
-        int tmpIndex = row + column * tmpRowDim;
-        final N tmpDenominator = tmpData[tmpIndex];
-
-        for (int i = row + 1; i < tmpRowDim; i++) {
-            tmpIndex++;
-            tmpDestination[i] = tmpData[tmpIndex] = tmpData[tmpIndex].divide(tmpDenominator).get();
+        for (int i = row + 1; i < myRowDim; i++) {
+            index++;
+            destinationData[i] = data[index] = data[index].divide(denominator).get();
         }
     }
 
-    public double doubleValue(final long row, final long col) {
+    @Override
+    public double doubleValue(final int row, final int col) {
         return this.doubleValue(row + col * myRowDim);
     }
 
@@ -558,10 +609,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         return true;
     }
 
+    @Override
     public void exchangeColumns(final long colA, final long colB) {
         myUtility.exchangeColumns(colA, colB);
     }
 
+    @Override
     public void exchangeHermitian(final int indexA, final int indexB) {
 
         final int tmpMin = Math.min(indexA, indexB);
@@ -591,10 +644,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void exchangeRows(final long rowA, final long rowB) {
         myUtility.exchangeRows(rowA, rowB);
     }
 
+    @Override
     public void fillByMultiplying(final Access1D<N> left, final Access1D<N> right) {
 
         final int complexity = Math.toIntExact(left.count() / this.countRows());
@@ -615,22 +670,27 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void fillColumn(final long row, final long col, final Access1D<N> values) {
         myUtility.fillColumn(row, col, values);
     }
 
+    @Override
     public void fillColumn(final long row, final long col, final N value) {
         myUtility.fillColumn(row, col, value);
     }
 
+    @Override
     public void fillColumn(final long row, final long col, final NullaryFunction<?> supplier) {
         myUtility.fillColumn(row, col, supplier);
     }
 
+    @Override
     public void fillDiagonal(final long row, final long col, final N value) {
         myUtility.fillDiagonal(row, col, value);
     }
 
+    @Override
     public void fillDiagonal(final long row, final long col, final NullaryFunction<?> supplier) {
         myUtility.fillDiagonal(row, col, supplier);
     }
@@ -734,58 +794,72 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void fillOne(final long row, final long col, final Access1D<?> values, final long valueIndex) {
         this.set(row, col, values.get(valueIndex));
     }
 
+    @Override
     public void fillOne(final long row, final long col, final N value) {
         myUtility.fillOne(row, col, value);
     }
 
+    @Override
     public void fillOne(final long row, final long col, final NullaryFunction<?> supplier) {
         myUtility.fillOne(row, col, supplier);
     }
 
+    @Override
     public void fillRow(final long row, final long col, final Access1D<N> values) {
         myUtility.fillRow(row, col, values);
     }
 
+    @Override
     public void fillRow(final long row, final long col, final N value) {
         myUtility.fillRow(row, col, value);
     }
 
+    @Override
     public void fillRow(final long row, final long col, final NullaryFunction<?> supplier) {
         myUtility.fillRow(row, col, supplier);
     }
 
+    @Override
     public boolean generateApplyAndCopyHouseholderColumn(final int row, final int column, final Householder<N> destination) {
         return GenerateApplyAndCopyHouseholderColumn.invoke(data, myRowDim, row, column, (Householder.Generic<N>) destination, myFactory.scalar());
     }
 
+    @Override
     public boolean generateApplyAndCopyHouseholderRow(final int row, final int column, final Householder<N> destination) {
         return GenerateApplyAndCopyHouseholderRow.invoke(data, myRowDim, row, column, (Householder.Generic<N>) destination, myFactory.scalar());
     }
 
+    @Override
     public MatrixStore<N> get() {
         return this;
     }
 
-    public N get(final long row, final long col) {
+    @Override
+    public N get(final int row, final int col) {
         return myUtility.get(row, col);
     }
 
+    @Override
     public int getColDim() {
         return myColDim;
     }
 
+    @Override
     public int getMaxDim() {
         return Math.max(myRowDim, myColDim);
     }
 
+    @Override
     public int getMinDim() {
         return Math.min(myRowDim, myColDim);
     }
 
+    @Override
     public int getRowDim() {
         return myRowDim;
     }
@@ -824,14 +898,17 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void modifyColumn(final long row, final long col, final UnaryFunction<N> modifier) {
         myUtility.modifyColumn(row, col, modifier);
     }
 
+    @Override
     public void modifyDiagonal(final long row, final long col, final UnaryFunction<N> modifier) {
         myUtility.modifyDiagonal(row, col, modifier);
     }
 
+    @Override
     public void modifyOne(final long row, final long col, final UnaryFunction<N> modifier) {
 
         N tmpValue = this.get(row, col);
@@ -841,10 +918,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         this.set(row, col, tmpValue);
     }
 
+    @Override
     public void modifyRow(final long row, final long col, final UnaryFunction<N> modifier) {
         myUtility.modifyRow(row, col, modifier);
     }
 
+    @Override
     public MatrixStore<N> multiply(final MatrixStore<N> right) {
 
         final GenericStore<N> retVal = this.physical().make(myRowDim, right.count() / myColDim);
@@ -858,6 +937,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         return retVal;
     }
 
+    @Override
     public N multiplyBoth(final Access1D<N> leftAndRight) {
 
         final PhysicalStore<N> tmpStep1 = myFactory.make(1L, leftAndRight.count());
@@ -872,67 +952,83 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         return tmpStep2.get(0L);
     }
 
+    @Override
     public void negateColumn(final int column) {
         myUtility.modifyColumn(0, column, myFactory.function().negate());
     }
 
+    @Override
     public PhysicalStore.Factory<N, GenericStore<N>> physical() {
         return myFactory;
     }
 
+    @Override
     public TransformableRegion<N> regionByColumns(final int... columns) {
         return new Subregion2D.ColumnsRegion<>(this, multiplyBoth, columns);
     }
 
+    @Override
     public TransformableRegion<N> regionByLimits(final int rowLimit, final int columnLimit) {
         return new Subregion2D.LimitRegion<>(this, multiplyBoth, rowLimit, columnLimit);
     }
 
+    @Override
     public TransformableRegion<N> regionByOffsets(final int rowOffset, final int columnOffset) {
         return new Subregion2D.OffsetRegion<>(this, multiplyBoth, rowOffset, columnOffset);
     }
 
+    @Override
     public TransformableRegion<N> regionByRows(final int... rows) {
         return new Subregion2D.RowsRegion<>(this, multiplyBoth, rows);
     }
 
+    @Override
     public TransformableRegion<N> regionByTransposing() {
         return new Subregion2D.TransposedRegion<>(this, multiplyBoth);
     }
 
+    @Override
     public void rotateRight(final int low, final int high, final double cos, final double sin) {
         RotateRight.invoke(data, myRowDim, low, high, myFactory.scalar().cast(cos), myFactory.scalar().cast(sin));
     }
 
+    @Override
+    public void set(final int row, final int col, final double value) {
+        myUtility.set(row, col, value);
+    }
+
+    @Override
     public void set(final long row, final long col, final Comparable<?> value) {
         myUtility.set(row, col, value);
     }
 
-    public void set(final long row, final long col, final double value) {
-        myUtility.set(row, col, value);
-    }
-
+    @Override
     public void setToIdentity(final int col) {
         myUtility.set(col, col, myFactory.scalar().one().get());
         myUtility.fillColumn(col + 1, col, myFactory.scalar().zero().get());
     }
 
+    @Override
     public Array1D<N> sliceColumn(final long row, final long col) {
         return myUtility.sliceColumn(row, col);
     }
 
+    @Override
     public Array1D<N> sliceDiagonal(final long row, final long col) {
         return myUtility.sliceDiagonal(row, col);
     }
 
+    @Override
     public Array1D<N> sliceRange(final long first, final long limit) {
         return myUtility.sliceRange(first, limit);
     }
 
+    @Override
     public Array1D<N> sliceRow(final long row, final long col) {
         return myUtility.sliceRow(row, col);
     }
 
+    @Override
     public void substituteBackwards(final Access2D<N> body, final boolean unitDiagonal, final boolean conjugated, final boolean hermitian) {
 
         final int tmpRowDim = myRowDim;
@@ -958,6 +1054,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void substituteForwards(final Access2D<N> body, final boolean unitDiagonal, final boolean conjugated, final boolean identity) {
 
         final int tmpRowDim = myRowDim;
@@ -982,6 +1079,7 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public Scalar<N> toScalar(final long row, final long column) {
         return myUtility.get(row, column);
     }
@@ -991,10 +1089,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         return Access2D.toString(this);
     }
 
+    @Override
     public void transformLeft(final Householder<N> transformation, final int firstColumn) {
         HouseholderLeft.call(data, myRowDim, firstColumn, this.cast(transformation), myFactory.scalar());
     }
 
+    @Override
     public void transformLeft(final Rotation<N> transformation) {
 
         final Rotation.Generic<N> tmpTransf = this.cast(transformation);
@@ -1017,10 +1117,12 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void transformRight(final Householder<N> transformation, final int firstRow) {
         HouseholderRight.call(data, myRowDim, firstRow, this.cast(transformation), myFactory.scalar());
     }
 
+    @Override
     public void transformRight(final Rotation<N> transformation) {
 
         final Rotation.Generic<N> tmpTransf = this.cast(transformation);
@@ -1043,38 +1145,44 @@ public final class GenericStore<N extends Scalar<N>> extends ScalarArray<N> impl
         }
     }
 
+    @Override
     public void transformSymmetric(final Householder<N> transformation) {
         HouseholderHermitian.invoke(data, this.cast(transformation), this.getWorkerColumn(), myFactory.scalar());
     }
 
+    @Override
     public MatrixStore<N> transpose() {
         return new TransposedStore<>(this);
     }
 
+    @Override
     public void tred2(final BasicArray<N> mainDiagonal, final BasicArray<N> offDiagonal, final boolean yesvecs) {
         ProgrammingError.throwForUnsupportedOptionalOperation();
     }
 
+    @Override
     public void visitColumn(final long row, final long col, final VoidFunction<N> visitor) {
         myUtility.visitColumn(row, col, visitor);
     }
 
+    @Override
     public void visitDiagonal(final long row, final long col, final VoidFunction<N> visitor) {
         myUtility.visitDiagonal(row, col, visitor);
     }
 
+    @Override
     public void visitRow(final long row, final long col, final VoidFunction<N> visitor) {
         myUtility.visitRow(row, col, visitor);
     }
 
-    private GenericStore<N> cast(final Access1D<N> matrix) {
+    private GenericStore<N> cast(final Access1D<?> matrix) {
         if (matrix instanceof GenericStore) {
             return (GenericStore<N>) matrix;
-        }
-        if (matrix instanceof Access2D<?>) {
+        } else if (matrix instanceof Access2D<?>) {
             return myFactory.copy((Access2D<?>) matrix);
+        } else {
+            return myFactory.columns(matrix);
         }
-        return myFactory.columns(matrix);
     }
 
     private Householder.Generic<N> cast(final Householder<N> transformation) {
