@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.math.BigDecimal;
 import java.util.*;
+import java.util.function.BiConsumer;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Predicate;
@@ -38,7 +39,6 @@ import java.util.stream.Stream;
 
 import org.ojalgo.ProgrammingError;
 import org.ojalgo.array.Array1D;
-import org.ojalgo.array.ArrayR064;
 import org.ojalgo.function.constant.BigMath;
 import org.ojalgo.netio.BasicLogger;
 import org.ojalgo.netio.InMemoryFile;
@@ -51,6 +51,8 @@ import org.ojalgo.structure.Access1D;
 import org.ojalgo.structure.Structure1D.IntIndex;
 import org.ojalgo.structure.Structure2D.IntRowColumn;
 import org.ojalgo.type.context.NumberContext;
+import org.ojalgo.type.keyvalue.EntryPair;
+import org.ojalgo.type.keyvalue.EntryPair.KeyedPrimitive;
 
 /**
  * <p>
@@ -87,12 +89,12 @@ import org.ojalgo.type.context.NumberContext;
  * <li>It has a presolver that tries to simplify the problem before invoking a solver (sometimes it turns out
  * there is no need to invoke a solver at all).</li>
  * <li>When/if needed it scales problem parameters, before creating solver specific data structures, to
- * minimize numerical problems in the solvers.</li>
+ * minimise numerical problems in the solvers.</li>
  * <li>It's the only way to access the integer solver.</li>
  * </ol>
  * <p>
  * Different solvers can be used, and ojAlgo comes with collection built in. The default built-in solvers can
- * handle anythimng you can model with a couple of restrictions:
+ * handle anything you can model with a couple of restrictions:
  * </p>
  * <ul>
  * <li>No quadratic constraints (The plan is that future versions should not have this limitation.)</li>
@@ -109,7 +111,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
      *
      * @author apete
      */
-    public static final class Description {
+    public static final class Description implements ProblemStructure {
 
         public final int nbEqualityBounds;
         public final int nbEqualityConstraints;
@@ -141,13 +143,94 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
             nbVariables = varTotal;
         }
 
-        /**
-         * The total number of constraints
-         */
-        public int constraints() {
+        @Override
+        public int countAdditionalConstraints() {
+            return 0;
+        }
+
+        @Override
+        public int countConstraints() {
             return nbEqualityConstraints + nbLowerConstraints + nbUpperConstraints;
         }
 
+        @Override
+        public int countEqualityConstraints() {
+            return nbEqualityBounds;
+        }
+
+        @Override
+        public int countInequalityConstraints() {
+            return nbLowerConstraints + nbUpperConstraints;
+        }
+
+        @Override
+        public int countVariables() {
+            return nbVariables;
+        }
+
+        @Override
+        public String toString() {
+            return "Description [nbEqualityBounds=" + nbEqualityBounds + ", nbEqualityConstraints=" + nbEqualityConstraints + ", nbIntegerVariables="
+                    + nbIntegerVariables + ", nbLowerBounds=" + nbLowerBounds + ", nbLowerConstraints=" + nbLowerConstraints + ", nbNegativeVariables="
+                    + nbNegativeVariables + ", nbPositiveVariables=" + nbPositiveVariables + ", nbUpperBounds=" + nbUpperBounds + ", nbUpperConstraints="
+                    + nbUpperConstraints + ", nbVariables=" + nbVariables + "]";
+        }
+
+    }
+
+    /**
+     * Connects solver constraints and variables back to model entities. Used for 2 things:
+     * <ol>
+     * <li>Solvers that manipulate models (like the {@link IntegerSolver}) need this to map between model
+     * entities and solver indices.
+     * <li>Simplifies implementation of
+     * {@link ExpressionsBasedModel.Integration#toModelState(org.ojalgo.optimisation.Optimisation.Result, ExpressionsBasedModel)}.
+     * </ol>
+     */
+    public static interface EntityMap extends ProblemStructure {
+
+        /**
+         * The number of variables, in the solver, that directly correspond to a model variable. (Not slack or
+         * artificial variables.) This defines the range of the indices that can be used with the indexOf
+         * method.
+         */
+        int countModelVariables();
+
+        /**
+         * The number of slack variables
+         */
+        int countSlackVariables();
+
+        EntryPair<ModelEntity<?>, ConstraintType> getConstraintMap(int i);
+
+        /**
+         * Returns which model entity, and constraint type, that corresponds to the slack variable at the
+         * supplied index.
+         *
+         * @param idx Index of solver slack variable (If there are 3 slack variables this input argument
+         *        should be in the range [0.2].)
+         */
+        EntryPair<ModelEntity<?>, ConstraintType> getSlack(int idx);
+
+        /**
+         * Converts from a solver specific variable index to the corresponding index of the variable in the
+         * model. Note that not all model variables are necessarily represented in the solver, and a model
+         * variable may result in multiple solver variables. Further, slack variables, artificial variables
+         * and such are typically not represented in the model.
+         *
+         * @param idx Index of solver variable
+         * @return Index of model variable (negative if no map)
+         */
+        int indexOf(int idx);
+
+        /**
+         * Is this solver variable negated relative to the corresponding model variable?
+         *
+         * @param idx Index of solver variable
+         * @return true if this solver variable represents a negated model variable
+         */
+        boolean isNegated(int idx);
+
     }
 
     public enum FileFormat {
@@ -190,76 +273,19 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         /**
          * @see Optimisation.Integration#extractSolverState(Optimisation.Model)
          */
+        @Override
         public final Result extractSolverState(final ExpressionsBasedModel model) {
             return this.toSolverState(model.getVariableValues(), model);
         }
 
-        /**
-         * The required behaviour here depends on how {@link #build(Optimisation.Model)} is implemented, and
-         * is the reverse mapping of {@link #toSolverState(Optimisation.Result, ExpressionsBasedModel)}.
-         *
-         * @see Optimisation.Integration#toModelState(Optimisation.Result, Optimisation.Model)
-         * @see #toSolverState(Optimisation.Result, ExpressionsBasedModel)
-         */
+        @Override
         public Result toModelState(final Result solverState, final ExpressionsBasedModel model) {
-
-            if (!this.isSolutionMapped()) {
-                return solverState;
-            }
-
-            List<Variable> freeVariables = model.getFreeVariables();
-            Set<IntIndex> fixedVariables = model.getFixedVariables();
-            int nbFreeVars = freeVariables.size();
-            int nbModelVars = model.countVariables();
-
-            ArrayR064 modelSolution = ArrayR064.make(nbModelVars);
-
-            for (int i = 0; i < nbFreeVars; i++) {
-                modelSolution.set(model.indexOf(freeVariables.get(i)), solverState.doubleValue(i));
-            }
-
-            for (IntIndex fixed : fixedVariables) {
-                modelSolution.set(fixed.index, model.getVariable(fixed.index).getValue());
-            }
-
-            return new Result(solverState.getState(), modelSolution);
+            return solverState;
         }
 
-        /**
-         * The required behaviour here depends on how {@link #build(Optimisation.Model)} is implemented!
-         * <p>
-         * If {@link #isSolutionMapped()} returns false this implementation does nothing – the input model
-         * state is returned as the solver state.
-         * <p>
-         * If {@link #isSolutionMapped()} returns true the standard mapping is applied - just accounting for
-         * that not all model variables are present in the solver. Variables that are fixed, by the presolver
-         * in the model, are typically not present in the solver.
-         * <p>
-         * If mapping needs to be performed, but it is not the standard mapping, then this method needs to be
-         * overridden for the specific implementation. For instance an LP simplex solver that separates
-         * between the positive and negative parts of variables need to do that. In that case
-         * {@link #isSolutionMapped()} is irrelevant, but should return true for good measure.
-         *
-         * @see Optimisation.Integration#toSolverState(Optimisation.Result, Optimisation.Model)
-         */
+        @Override
         public Result toSolverState(final Result modelState, final ExpressionsBasedModel model) {
-
-            if (!this.isSolutionMapped()) {
-                return modelState;
-            }
-
-            List<Variable> freeVariables = model.getFreeVariables();
-            int nbFreeVars = freeVariables.size();
-
-            ArrayR064 solverSolution = ArrayR064.make(nbFreeVars);
-
-            for (int i = 0; i < nbFreeVars; i++) {
-                Variable variable = freeVariables.get(i);
-                int modelIndex = model.indexOf(variable);
-                solverSolution.set(i, modelState.doubleValue(modelIndex));
-            }
-
-            return new Result(modelState.getState(), solverSolution);
+            return modelState;
         }
 
         /**
@@ -286,12 +312,17 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
             return model.indexOfFreeVariable(variable);
         }
 
-        /**
-         * @return true if the set of variables present in the solver is not precisely the same as in the
-         *         model. If fixed variables are omitted or if variables are split into a positive and
-         *         negative part, then this method must return true
-         */
-        protected abstract boolean isSolutionMapped();
+        protected final boolean isSwitch(final ExpressionsBasedModel model) {
+            return model.isIntegrationSwitch();
+        }
+
+        protected final ExpressionsBasedModel.Validator newValidator(final ExpressionsBasedModel model) {
+            return new ExpressionsBasedModel.Validator(model, this, model.getKnownSolution(), model.getValidationFailureHandler());
+        }
+
+        protected final void setSwitch(final ExpressionsBasedModel model, final boolean value) {
+            model.setIntegrationSwitch(value);
+        }
 
     }
 
@@ -317,6 +348,88 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
 
     }
 
+    public static final class Validator {
+
+        private static final NumberContext ACCURACY = NumberContext.of(8);
+
+        static final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> NULL = (m, s) -> {};
+
+        public static Validator of(final ExpressionsBasedModel originalModel, final Optimisation.Integration<ExpressionsBasedModel, ?> integration) {
+            return Validator.of(originalModel, integration, NULL);
+        }
+
+        public static Validator of(final ExpressionsBasedModel originalModel, final Optimisation.Integration<ExpressionsBasedModel, ?> integration,
+                final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> handler) {
+            Objects.requireNonNull(originalModel);
+            Objects.requireNonNull(integration);
+            return new Validator(originalModel, integration, null, handler);
+        }
+
+        public static Validator of(final Result knownSolution) {
+            Validator.of(knownSolution, NULL);
+            Objects.requireNonNull(knownSolution);
+            return new Validator(null, null, knownSolution, NULL);
+        }
+
+        public static Validator of(final Result knownSolution, final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> handler) {
+            Objects.requireNonNull(knownSolution);
+            return new Validator(null, null, knownSolution, handler);
+        }
+
+        private static boolean doValidate(final ExpressionsBasedModel model, final Access1D<BigDecimal> solution, final NumberContext accuracy,
+                final BasicLogger logger, final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> handler) {
+
+            if (model != null && solution != null) {
+                boolean valid = model.validate(solution, accuracy != null ? accuracy : ACCURACY, logger != null ? logger : BasicLogger.NULL);
+                if (!valid) {
+                    handler.accept(model, solution);
+                }
+                return valid;
+            } else {
+                return true;
+            }
+        }
+
+        private final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> myHandler;
+        private final Optimisation.Integration<ExpressionsBasedModel, ?> myIntegration;
+        private final Optimisation.Result myKnownSolution;
+        private final ExpressionsBasedModel myOriginalModel;
+
+        /**
+         * @param originalModel Baseline model.
+         * @param integration The integration used to translate between model and solver state.
+         * @param knownSolution Not just any feasible solution. It needs to be the optimal solution.
+         */
+        Validator(final ExpressionsBasedModel originalModel, final Optimisation.Integration<ExpressionsBasedModel, ?> integration, final Result knownSolution,
+                final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> handler) {
+            super();
+            myOriginalModel = originalModel;
+            myIntegration = integration;
+            myKnownSolution = knownSolution;
+            myHandler = handler;
+        }
+
+        /**
+         * Validate an (intermediate) solver solution against the original model. (Validation only performed
+         * if an original model was provided to the constructor.)
+         */
+        public boolean validate(final Access1D<?> solverSolution, final NumberContext accuracy, final BasicLogger logger) {
+            Result solverState = Optimisation.Result.wrap(solverSolution);
+            Result modelState = myIntegration.toModelState(solverState, myOriginalModel);
+            return Validator.doValidate(myOriginalModel, modelState, accuracy, logger, myHandler);
+        }
+
+        /**
+         * Validate the known solution against a (modified) model - perhaps modified during pre-solve or with
+         * cuts generation in the {@link IntegerSolver}. (Validation only performed if a known solution was
+         * provided to the constructor.)
+         */
+        public boolean validate(final ExpressionsBasedModel modifiedModel, final NumberContext accuracy, final BasicLogger logger) {
+            return Validator.doValidate(modifiedModel, myKnownSolution, accuracy, logger, myHandler);
+        }
+
+    }
+
     static final class DefaultIntermediate extends IntermediateSolver {
 
         DefaultIntermediate(final ExpressionsBasedModel model) {
@@ -335,6 +448,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
             myExecutionOrder = executionOrder;
         }
 
+        @Override
         public final int compareTo(final S reference) {
             return Integer.compare(myExecutionOrder, reference.getExecutionOrder());
         }
@@ -534,9 +648,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
     static final TreeSet<Presolver> PRESOLVERS = new TreeSet<>();
 
     static {
-        ExpressionsBasedModel.addPresolver(Presolvers.ZERO_ONE_TWO);
-        ExpressionsBasedModel.addPresolver(Presolvers.INTEGER);
-        ExpressionsBasedModel.addPresolver(Presolvers.REDUNDANT_CONSTRAINT);
+        ExpressionsBasedModel.resetPresolvers();
     }
 
     /**
@@ -593,11 +705,19 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return PRESOLVERS.remove(presolver);
     }
 
+    public static void resetPresolvers() {
+        ExpressionsBasedModel.addPresolver(Presolvers.ZERO_ONE_TWO);
+        ExpressionsBasedModel.addPresolver(Presolvers.INTEGER);
+        ExpressionsBasedModel.addPresolver(Presolvers.REDUNDANT_CONSTRAINT);
+    }
+
     public final Optimisation.Options options;
 
     private final Map<String, Expression> myExpressions = new HashMap<>();
     private final Set<IntIndex> myFixedVariables = new HashSet<>();
     private transient boolean myInfeasible = false;
+    private boolean myIntegrationSwitch = false;
+    private Optimisation.Result myKnownSolution = null;
     private BigDecimal myObjectiveConstant = BigMath.ZERO;
     private Optimisation.Sense myOptimisationSense = null;
     private final Set<IntIndex> myReferences;
@@ -608,9 +728,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
      */
     private final boolean myShallowCopy;
     /**
-     * Temporary storage for some expresssion specific subset of variables
+     * Temporary storage for some expression specific subset of variables
      */
     private final Set<IntIndex> myTemporary = new HashSet<>();
+    private BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> myValidationFailureHandler = Validator.NULL;
     private final ArrayList<Variable> myVariables = new ArrayList<>();
     private final VariablesCategorisation myVariablesCategorisation = new VariablesCategorisation();
 
@@ -618,6 +739,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         this(new Optimisation.Options());
     }
 
+    /**
+     * @deprecated v53 Use {@link #ExpressionsBasedModel()} and {@link #newVariable(String)} instead.
+     */
+    @Deprecated
     public ExpressionsBasedModel(final Collection<? extends Variable> variables) {
 
         this();
@@ -639,6 +764,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         myRelaxed = false;
     }
 
+    /**
+     * @deprecated v53 Use {@link #ExpressionsBasedModel()} and {@link #newVariable(String)} instead.
+     */
+    @Deprecated
     public ExpressionsBasedModel(final Variable... variables) {
 
         this();
@@ -688,16 +817,11 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
     }
 
     public Expression addExpression() {
-        return this.addExpression("EXPR" + myExpressions.size());
+        return this.newExpression("EXPR" + myExpressions.size());
     }
 
     public Expression addExpression(final String name) {
-
-        final Expression retVal = new Expression(name, this);
-
-        myExpressions.put(name, retVal);
-
-        return retVal;
+        return this.newExpression(name);
     }
 
     /**
@@ -751,7 +875,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
 
         final String name = "SOS" + max + "-" + orderedSet.toString();
 
-        final Expression expression = this.addExpression(name);
+        final Expression expression = this.newExpression(name);
 
         for (final Variable variable : orderedSet) {
             if (variable == null || variable.getIndex() == null || !variable.isBinary()) {
@@ -769,15 +893,17 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
     }
 
     public Variable addVariable() {
-        return this.addVariable("X" + myVariables.size());
+        return this.newVariable("X" + myVariables.size());
     }
 
     public Variable addVariable(final String name) {
-        final Variable retVal = new Variable(name);
-        this.addVariable(retVal);
-        return retVal;
+        return this.newVariable(name);
     }
 
+    /**
+     * @deprecated v53 Use {@link #newVariable(String)} instead.
+     */
+    @Deprecated
     public void addVariable(final Variable variable) {
         if (myShallowCopy) {
             throw new IllegalStateException("This model is a work copy - its set of variables cannot be modified!");
@@ -786,12 +912,20 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         variable.setIndex(new IntIndex(myVariables.size() - 1));
     }
 
+    /**
+     * @deprecated v53 Use {@link #newVariable(String)} instead.
+     */
+    @Deprecated
     public void addVariables(final Collection<? extends Variable> variables) {
         for (final Variable tmpVariable : variables) {
             this.addVariable(tmpVariable);
         }
     }
 
+    /**
+     * @deprecated v53 Use {@link #newVariable(String)} instead.
+     */
+    @Deprecated
     public void addVariables(final Variable[] variables) {
         for (final Variable tmpVariable : variables) {
             this.addVariable(tmpVariable);
@@ -994,7 +1128,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
 
         State retState = State.UNEXPLORED;
         double retValue = Double.NaN;
-        final Array1D<BigDecimal> retSolution = Array1D.R128.make(numberOfVariables);
+        final Array1D<BigDecimal> retSolution = Array1D.R256.make(numberOfVariables);
 
         boolean allVarsSomeInfo = true;
 
@@ -1177,6 +1311,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return constrExpr != null ? constrExpr : this.objective();
     }
 
+    @Override
     public Optimisation.Result maximise() {
 
         this.setOptimisationSense(Optimisation.Sense.MAX);
@@ -1184,6 +1319,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return this.optimise();
     }
 
+    @Override
     public Optimisation.Result minimise() {
 
         this.setOptimisationSense(Optimisation.Sense.MIN);
@@ -1191,6 +1327,21 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return this.optimise();
     }
 
+    public Expression newExpression(final String name) {
+
+        final Expression retVal = new Expression(name, this);
+
+        myExpressions.put(name, retVal);
+
+        return retVal;
+    }
+
+    public Variable newVariable(final String name) {
+        final Variable retVal = new Variable(name);
+        this.addVariable(retVal);
+        return retVal;
+    }
+
     /**
      * This is generated on demand – you should not cache this. More specifically, modifications made to this
      * expression will not be part of the optimisation model. You define the objective by setting the
@@ -1306,6 +1457,33 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         myExpressions.remove(name);
     }
 
+    /**
+     * Same as {@link #setKnownSolution(org.ojalgo.optimisation.Optimisation.Result, BiConsumer)} but with a
+     * no-op handler.
+     */
+    public void setKnownSolution(final Optimisation.Result knownSolution) {
+        this.setKnownSolution(knownSolution, Validator.NULL);
+    }
+
+    /**
+     * For test/validation during solver development.
+     * 
+     * @param knownSolution The optimal solution
+     * @param handler What to do if validation fails
+     */
+    public void setKnownSolution(final Optimisation.Result knownSolution, final BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> handler) {
+        Objects.requireNonNull(knownSolution);
+        Objects.requireNonNull(handler);
+        if (!knownSolution.getState().isOptimal()) {
+            throw new ProgrammingError("Must be an optimal solution!");
+        }
+        if (!this.validate(knownSolution)) {
+            throw new ProgrammingError("Solution not valid!");
+        }
+        myKnownSolution = knownSolution;
+        myValidationFailureHandler = handler;
+    }
+
     /**
      * Will perform presolve and then create a copy removing redundant constraint expressions, and pruning the
      * remaining ones to no longer include fixed variables.
@@ -1333,13 +1511,14 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
 
         StringBuilder retVal = new StringBuilder(START_END);
 
-        for (Variable tmpVariable : myVariables) {
-            tmpVariable.appendToString(retVal);
+        for (Variable variable : myVariables) {
+            variable.appendToString(retVal, options.print);
             retVal.append(NEW_LINE);
         }
 
-        for (Expression tmpExpression : myExpressions.values()) {
-            tmpExpression.appendToString(retVal, this.getVariableValues());
+        Result solution = this.getVariableValues();
+        for (Expression expression : myExpressions.values()) {
+            expression.appendToString(retVal, solution, options.print);
             retVal.append(NEW_LINE);
         }
 
@@ -1352,6 +1531,7 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
      *
      * @see Optimisation.Model#validate()
      */
+    @Override
     public boolean validate() {
 
         final BasicLogger appender = options.logger_detailed ? options.logger_appender : BasicLogger.NULL;
@@ -1476,10 +1656,11 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         Result retSolution = this.getVariableValues();
         double retValue = this.objective().evaluate(retSolution).doubleValue();
         Optimisation.State retState = result.getState();
+        List<KeyedPrimitive<EntryPair<ModelEntity<?>, ConstraintType>>> matchedMultipliers = result.getMatchedMultipliers();
 
         prepared.dispose();
 
-        return new Optimisation.Result(retState, retValue, retSolution);
+        return new Optimisation.Result(retState, retValue, retSolution).multipliers(matchedMultipliers);
     }
 
     private void scanEntities() {
@@ -1572,6 +1753,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return retVal;
     }
 
+    Optimisation.Result getKnownSolution() {
+        return myKnownSolution;
+    }
+
     BigDecimal getObjectiveConstant() {
         return myObjectiveConstant;
     }
@@ -1580,6 +1765,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return myReferences;
     }
 
+    BiConsumer<ExpressionsBasedModel, Access1D<BigDecimal>> getValidationFailureHandler() {
+        return myValidationFailureHandler != null ? myValidationFailureHandler : Validator.NULL;
+    }
+
     boolean isFixed() {
         return myVariables.stream().allMatch(Variable::isFixed);
     }
@@ -1616,6 +1805,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         return true;
     }
 
+    boolean isIntegrationSwitch() {
+        return myIntegrationSwitch;
+    }
+
     boolean isReferenced(final Variable variable) {
         return myReferences.contains(variable.getIndex());
     }
@@ -1701,6 +1894,10 @@ public final class ExpressionsBasedModel implements Optimisation.Model {
         myInfeasible = true;
     }
 
+    void setIntegrationSwitch(final boolean value) {
+        myIntegrationSwitch = value;
+    }
+
     void setOptimisationSense(final Optimisation.Sense optimisationSense) {
         myOptimisationSense = optimisationSense;
     }
