@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -45,19 +45,14 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
     private static final BigDecimal LARGEST = new BigDecimal(Double.toString(PrimitiveMath.MACHINE_LARGEST), new MathContext(8, RoundingMode.DOWN));
     private static final BigDecimal SMALLEST = new BigDecimal(Double.toString(PrimitiveMath.MACHINE_SMALLEST), new MathContext(8, RoundingMode.UP));
 
-    static final NumberContext DISPLAY = NumberContext.ofScale(6);
+    static final NumberContext PRINT = NumberContext.of(6);
+    static final int RANGE = 8;
 
     static int deriveAdjustmentExponent(final AggregatorFunction<BigDecimal> largest, final AggregatorFunction<BigDecimal> smallest, final int range) {
 
         double expL = MissingMath.log10(largest.doubleValue(), PrimitiveMath.ZERO);
 
-        int doubleRange = 2 * range;
-
-        if (expL > doubleRange) {
-            return 0;
-        }
-
-        double expS = Math.max(MissingMath.log10(smallest.doubleValue(), -doubleRange), expL - range);
+        double expS = Math.max(MissingMath.log10(smallest.doubleValue(), -range), expL - range);
 
         double negatedAverage = (expL + expS) / -PrimitiveMath.TWO;
 
@@ -122,6 +117,12 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         ProgrammingError.throwIfNull(name);
     }
 
+    /**
+     * Add this ({@link Variable} or {@link Expression}) to another {@link Expression}, scaled by a factor.
+     * 
+     * @param target The target {@link Expression}
+     * @param scale The scaling factor
+     */
     public abstract void addTo(Expression target, BigDecimal scale);
 
     public final BigDecimal adjust(final BigDecimal factor) {
@@ -136,7 +137,7 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
 
         boolean retVal = false;
 
-        if ((obj instanceof ModelEntity<?>) && myName.equals(((ModelEntity<?>) obj).getName())) {
+        if (obj instanceof ModelEntity<?> && myName.equals(((ModelEntity<?>) obj).getName())) {
             retVal = true;
         }
 
@@ -166,33 +167,36 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return BigDecimal.ONE.movePointRight(this.getAdjustmentExponent()).doubleValue(); // 10^exponent
     }
 
+    @Override
     public final BigDecimal getContributionWeight() {
         return myContributionWeight;
     }
 
+    @Override
     public final BigDecimal getLowerLimit() {
         return myLowerLimit;
     }
 
+    public final BigDecimal getLowerLimit(final boolean adjusted, final BigDecimal defaultValue) {
+
+        BigDecimal limit = this.getLower(adjusted);
+
+        if (limit != null) {
+            return limit;
+        } else {
+            return defaultValue;
+        }
+    }
+
     public final double getLowerLimit(final boolean adjusted, final double defaultValue) {
 
-        BigDecimal limit = null;
-        if (adjusted && myLowerLimit != null) {
-            int adjustmentExponent = this.getAdjustmentExponent();
-            if (adjustmentExponent != 0) {
-                limit = myLowerLimit.movePointRight(adjustmentExponent);
-            } else {
-                limit = myLowerLimit;
-            }
-        } else {
-            limit = myLowerLimit;
-        }
+        BigDecimal limit = this.getLower(adjusted);
 
         if (limit != null) {
             return limit.doubleValue();
+        } else {
+            return defaultValue;
         }
-
-        return defaultValue;
     }
 
     public final String getName() {
@@ -215,29 +219,31 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return this.getUpperLimit(false, Double.POSITIVE_INFINITY);
     }
 
+    @Override
     public final BigDecimal getUpperLimit() {
         return myUpperLimit;
     }
 
+    public final BigDecimal getUpperLimit(final boolean adjusted, final BigDecimal defaultValue) {
+
+        BigDecimal limit = this.getUpper(adjusted);
+
+        if (limit != null) {
+            return limit;
+        } else {
+            return defaultValue;
+        }
+    }
+
     public final double getUpperLimit(final boolean adjusted, final double defaultValue) {
 
-        BigDecimal limit = null;
-        if (adjusted && myUpperLimit != null) {
-            int adjustmentExponent = this.getAdjustmentExponent();
-            if (adjustmentExponent != 0) {
-                limit = myUpperLimit.movePointRight(adjustmentExponent);
-            } else {
-                limit = myUpperLimit;
-            }
-        } else {
-            limit = myUpperLimit;
-        }
+        BigDecimal limit = this.getUpper(adjusted);
 
         if (limit != null) {
             return limit.doubleValue();
+        } else {
+            return defaultValue;
         }
-
-        return defaultValue;
     }
 
     @Override
@@ -245,6 +251,7 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return myName.hashCode();
     }
 
+    @Override
     public final boolean isConstraint() {
         return myLowerLimit != null || myUpperLimit != null;
     }
@@ -253,6 +260,7 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return myContributionWeight != null;
     }
 
+    @Override
     public final boolean isEqualityConstraint() {
         return myLowerLimit != null && myUpperLimit != null && myLowerLimit.compareTo(myUpperLimit) == 0;
     }
@@ -262,6 +270,7 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
      */
     public abstract boolean isInteger();
 
+    @Override
     public final boolean isLowerConstraint() {
         return myLowerLimit != null && !this.isEqualityConstraint();
     }
@@ -270,10 +279,12 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return myLowerLimit != null;
     }
 
+    @Override
     public final boolean isObjective() {
         return myContributionWeight != null && myContributionWeight.signum() != 0;
     }
 
+    @Override
     public final boolean isUpperConstraint() {
         return myUpperLimit != null && !this.isEqualityConstraint();
     }
@@ -369,9 +380,9 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
     @Override
     public final String toString() {
 
-        final StringBuilder retVal = new StringBuilder();
+        StringBuilder retVal = new StringBuilder();
 
-        this.appendToString(retVal);
+        this.appendToString(retVal, PRINT);
 
         return retVal.toString();
     }
@@ -439,28 +450,58 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return this.weight(BigDecimal.valueOf(weight));
     }
 
-    protected void appendLeftPart(final StringBuilder builder) {
+    private BigDecimal getLower(final boolean adjusted) {
+        BigDecimal limit = null;
+        if (adjusted && myLowerLimit != null) {
+            int adjustmentExponent = this.getAdjustmentExponent();
+            if (adjustmentExponent != 0) {
+                limit = myLowerLimit.movePointRight(adjustmentExponent);
+            } else {
+                limit = myLowerLimit;
+            }
+        } else {
+            limit = myLowerLimit;
+        }
+        return limit;
+    }
+
+    private BigDecimal getUpper(final boolean adjusted) {
+        BigDecimal limit = null;
+        if (adjusted && myUpperLimit != null) {
+            int adjustmentExponent = this.getAdjustmentExponent();
+            if (adjustmentExponent != 0) {
+                limit = myUpperLimit.movePointRight(adjustmentExponent);
+            } else {
+                limit = myUpperLimit;
+            }
+        } else {
+            limit = myUpperLimit;
+        }
+        return limit;
+    }
+
+    protected void appendLeftPart(final StringBuilder builder, final NumberContext display) {
         if (this.isLowerConstraint() || this.isEqualityConstraint()) {
-            builder.append(ModelEntity.DISPLAY.enforce(this.getLowerLimit()).toPlainString());
+            builder.append(display.enforce(this.getLowerLimit()).toPlainString());
             builder.append(" <= ");
         }
     }
 
-    protected void appendMiddlePart(final StringBuilder builder) {
+    protected void appendMiddlePart(final StringBuilder builder, final NumberContext display) {
 
         builder.append(this.getName());
 
         if (this.isObjective()) {
             builder.append(" (");
-            builder.append(ModelEntity.DISPLAY.enforce(this.getContributionWeight()).toPlainString());
+            builder.append(display.enforce(this.getContributionWeight()).toPlainString());
             builder.append(")");
         }
     }
 
-    protected void appendRightPart(final StringBuilder builder) {
+    protected void appendRightPart(final StringBuilder builder, final NumberContext display) {
         if (this.isUpperConstraint() || this.isEqualityConstraint()) {
             builder.append(" <= ");
-            builder.append(ModelEntity.DISPLAY.enforce(this.getUpperLimit()).toPlainString());
+            builder.append(display.enforce(this.getUpperLimit()).toPlainString());
         }
     }
 
@@ -484,7 +525,7 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
 
         boolean retVal = true;
 
-        if ((myLowerLimit != null && myUpperLimit != null) && (myLowerLimit.compareTo(myUpperLimit) > 0 || myUpperLimit.compareTo(myLowerLimit) < 0)) {
+        if (myLowerLimit != null && myUpperLimit != null && (myLowerLimit.compareTo(myUpperLimit) > 0 || myUpperLimit.compareTo(myLowerLimit) < 0)) {
             if (appender != null) {
                 appender.println(this.toString() + " The lower limit (if it exists) must be smaller than or equal to the upper limit (if it exists)!");
             }
@@ -526,10 +567,10 @@ public abstract class ModelEntity<ME extends ModelEntity<ME>> implements Optimis
         return retVal;
     }
 
-    final void appendToString(final StringBuilder builder) {
-        this.appendLeftPart(builder);
-        this.appendMiddlePart(builder);
-        this.appendRightPart(builder);
+    final void appendToString(final StringBuilder builder, final NumberContext display) {
+        this.appendLeftPart(builder, display);
+        this.appendMiddlePart(builder, display);
+        this.appendRightPart(builder, display);
     }
 
     abstract int deriveAdjustmentExponent();
