@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,20 +23,25 @@ package org.ojalgo.optimisation;
 
 import java.math.BigDecimal;
 import java.math.RoundingMode;
+import java.util.List;
 import java.util.Objects;
 import java.util.Optional;
 
 import org.ojalgo.ProgrammingError;
 import org.ojalgo.array.ArrayR064;
-import org.ojalgo.array.ArrayR128;
+import org.ojalgo.array.ArrayR256;
 import org.ojalgo.netio.BasicLogger;
+import org.ojalgo.optimisation.convex.ConvexSolver;
 import org.ojalgo.optimisation.integer.IntegerSolver;
 import org.ojalgo.optimisation.integer.IntegerStrategy;
+import org.ojalgo.optimisation.linear.LinearSolver;
 import org.ojalgo.structure.Access1D;
 import org.ojalgo.type.CalendarDateDuration;
 import org.ojalgo.type.CalendarDateUnit;
 import org.ojalgo.type.TypeUtils;
 import org.ojalgo.type.context.NumberContext;
+import org.ojalgo.type.keyvalue.EntryPair;
+import org.ojalgo.type.keyvalue.EntryPair.KeyedPrimitive;
 
 public interface Optimisation {
 
@@ -59,7 +64,7 @@ public interface Optimisation {
 
         /**
          * The Constraint has a lower or an upper limit actually set (possibly both) - it actually is
-         * constained.
+         * constrained.
          */
         boolean isConstraint();
 
@@ -134,6 +139,9 @@ public interface Optimisation {
          * Convert solver state to model state. Transforming the solution (set of variable values) is the main
          * concern. Adjusting the objective function value (if needed) is best handled elsewhere, and is not
          * required here.
+         * <p>
+         * The required behaviour here depends on how {@link #build(Optimisation.Model)} is implemented, and
+         * is the reverse mapping of {@link #toSolverState(Optimisation.Result, Optimisation.Model)}.
          */
         Optimisation.Result toModelState(Optimisation.Result solverState, M model);
 
@@ -141,6 +149,9 @@ public interface Optimisation {
          * Convert model state to solver state. Transforming the solution (set of variable values) is the main
          * concern. Adjusting the objective function value (if needed) is best handled elsewhere, and is not
          * required here.
+         * <p>
+         * The required behaviour here depends on how {@link #build(Optimisation.Model)} is implemented, and
+         * is the reverse mapping of {@link #toModelState(Result, Optimisation.Model)}.
          */
         Optimisation.Result toSolverState(Optimisation.Result modelState, M model);
 
@@ -191,8 +202,17 @@ public interface Optimisation {
     public static final class Options implements Optimisation {
 
         /**
-         * Used to determine/validate feasibility. Are the constraints violated or not? Are the variable
-         * values integer or not?
+         * This may turn on various experimental features. If you do not know exactly what you want to turn
+         * on, for the specific version you're using, then always leave this 'false'.
+         */
+        public boolean experimental = false;
+
+        /**
+         * Used to determine/validate feasibility. Are the variables within their bounds or not, are the
+         * constraints violated or not? are the variable values integer or not?
+         * <p>
+         * Primarily used in {@link ExpressionsBasedModel}. Not used (should not be) as part of solver logic,
+         * but ouside the solvers to validate their results.
          */
         public NumberContext feasibility = NumberContext.of(12, 8);
 
@@ -227,28 +247,15 @@ public interface Optimisation {
         public Class<? extends Optimisation.Solver> logger_solver = null;
 
         /**
-         * @deprecated v51.1.0 No longer used for anything! Instead it is possible to specify an
-         *             {@link IntegerStrategy} that offers much more control.
+         * For display only! {@link #toString()} and log message formatting.
          */
-        @Deprecated
-        public double mip_defer = 0.99;
-
-        /**
-         * @deprecated v51.1.0 No longer used! Use {@link IntegerStrategy#getGapTolerance()} instead.
-         */
-        @Deprecated
-        public double mip_gap = 1.0E-6;
-
-        /**
-         * For display only!
-         */
-        public NumberContext print = NumberContext.of(8, 10);
+        public NumberContext print = ModelEntity.PRINT;
 
         /**
          * Describes the (required/sufficient) accuracy of the solution. It is used when copying the solver's
          * solution back to the model (converting from double to BigDecimal). Specific solvers may also use
          * this as a stopping criteria or similar. The default essentially copies the numbers as is –
-         * corresponding to full double precision.
+         * corresponding to full double precision – but with no more than 14 decimals.
          */
         public NumberContext solution = NumberContext.ofScale(14).withMode(RoundingMode.HALF_DOWN);
 
@@ -291,7 +298,9 @@ public interface Optimisation {
         public boolean validate = false;
 
         private Object myConfigurator = null;
+        private ConvexSolver.Configuration myConvexConfiguration = new ConvexSolver.Configuration();
         private IntegerStrategy myIntegerStrategy = IntegerStrategy.DEFAULT;
+        private LinearSolver.Configuration myLinearConfiguration = new LinearSolver.Configuration();
 
         public Options() {
             super();
@@ -303,14 +312,28 @@ public interface Optimisation {
             return this;
         }
 
+        /**
+         * Configurations specific to ojAlgo's built-in {@link ConvexSolver}.
+         */
+        public ConvexSolver.Configuration convex() {
+            return myConvexConfiguration;
+        }
+
+        public Options convex(final ConvexSolver.Configuration configuration) {
+            Objects.requireNonNull(configuration);
+            myConvexConfiguration = configuration;
+            return this;
+        }
+
         /**
          * Will configure detailed debug logging and validation
          */
-        public void debug(final Class<? extends Optimisation.Solver> solver) {
+        public Options debug(final Class<? extends Optimisation.Solver> solver) {
             logger_solver = solver;
             logger_appender = solver != null ? BasicLogger.DEBUG : null;
-            logger_detailed = (solver != null);
-            validate = (solver != null);
+            logger_detailed = solver != null;
+            validate = solver != null;
+            return this;
         }
 
         public <T> Optional<T> getConfigurator(final Class<T> type) {
@@ -335,16 +358,33 @@ public interface Optimisation {
             return this;
         }
 
+        public LinearSolver.Configuration linear() {
+            return myLinearConfiguration;
+        }
+
+        /**
+         * Configurations specific to ojAlgo's built-in {@link LinearSolver}.
+         */
+        public Options linear(final LinearSolver.Configuration configuration) {
+            Objects.requireNonNull(configuration);
+            myLinearConfiguration = configuration;
+            return this;
+        }
+
         /**
          * Will configure high level (low volume) progress logging
          */
-        public void progress(final Class<? extends Optimisation.Solver> solver) {
+        public Options progress(final Class<? extends Optimisation.Solver> solver) {
             logger_solver = solver;
             logger_appender = solver != null ? BasicLogger.DEBUG : null;
             logger_detailed = false;
             validate = false;
+            return this;
         }
 
+        /**
+         * A configurator for 3:d party solvers. Each such solver may define its own configurator type.
+         */
         public void setConfigurator(final Object configurator) {
             ProgrammingError.throwIfNull(configurator);
             myConfigurator = configurator;
@@ -358,6 +398,31 @@ public interface Optimisation {
 
     }
 
+    /**
+     * Basic description of the size/structure of an optimisation problem.
+     */
+    public interface ProblemStructure extends Optimisation {
+
+        static final boolean DEBUG = false;
+
+        /**
+         * Not included in {@link #countConstraints()} (because they are not simple linear equality or
+         * inequality constraints),
+         */
+        int countAdditionalConstraints();
+
+        default int countConstraints() {
+            return this.countEqualityConstraints() + this.countInequalityConstraints();
+        }
+
+        int countEqualityConstraints();
+
+        int countInequalityConstraints();
+
+        int countVariables();
+
+    }
+
     public static final class Result implements Optimisation, Access1D<BigDecimal>, Comparable<Optimisation.Result> {
 
         public static Result of(final double value, final Optimisation.State state, final double... solution) {
@@ -382,7 +447,7 @@ public interface Optimisation {
 
             State state = Optimisation.State.valueOf(strState);
             double value = Double.parseDouble(strValue);
-            ArrayR128 solution = ArrayR128.make(strSolution.length);
+            ArrayR256 solution = ArrayR256.make(strSolution.length);
             for (int i = 0; i < strSolution.length; i++) {
                 solution.set(i, new BigDecimal(strSolution[i]));
             }
@@ -390,7 +455,13 @@ public interface Optimisation {
             return new Result(state, value, solution);
         }
 
-        private transient Access1D<?> myMultipliers = null;
+        public static Result wrap(final Access1D<?> solution) {
+            return new Result(State.APPROXIMATE, Double.NaN, solution);
+        }
+
+        private ConstraintsMap myConstraintsMap = null;
+        private List<KeyedPrimitive<EntryPair<ModelEntity<?>, ConstraintType>>> myMatchedMultipliers = null;
+        private Access1D<?> myMultipliers = null;
         private final Access1D<?> mySolution;
         private final Optimisation.State myState;
         private final double myValue; // Objective Function Value
@@ -414,15 +485,18 @@ public interface Optimisation {
             this(state, result.getValue(), result);
         }
 
+        @Override
         public int compareTo(final Result reference) {
             return NumberContext.compare(myValue, reference.getValue());
         }
 
+        @Override
         public long count() {
             return mySolution.count();
         }
 
-        public double doubleValue(final long index) {
+        @Override
+        public double doubleValue(final int index) {
             return mySolution.doubleValue(index);
         }
 
@@ -441,10 +515,26 @@ public interface Optimisation {
             return true;
         }
 
+        @Override
         public BigDecimal get(final long index) {
             return TypeUtils.toBigDecimal(mySolution.get(index));
         }
 
+        /**
+         * The dual variables or Lagrange multipliers, matched to their respective constraints (model entity
+         * and constraint type pairs).
+         */
+        public List<KeyedPrimitive<EntryPair<ModelEntity<?>, ConstraintType>>> getMatchedMultipliers() {
+            if (myMatchedMultipliers == null) {
+                if (myConstraintsMap != null && myMultipliers != null && myConstraintsMap.isEntityMap() && myConstraintsMap.size() == myMultipliers.size()) {
+                    myMatchedMultipliers = myConstraintsMap.match(myMultipliers);
+                } else {
+                    myMatchedMultipliers = List.of();
+                }
+            }
+            return myMatchedMultipliers;
+        }
+
         /**
          * The dual variables or Lagrange multipliers associated with the problem.
          */
@@ -456,13 +546,13 @@ public interface Optimisation {
          * Will round the solution to the given precision
          */
         public Optimisation.Result getSolution(final NumberContext precision) {
-            Optimisation.State state = this.getState();
-            double value = this.getValue();
-            ArrayR128 solution = ArrayR128.make(this.size());
+
+            ArrayR256 solution = ArrayR256.make(this.size());
             for (int i = 0, limit = solution.data.length; i < limit; i++) {
                 solution.set(i, precision.enforce(this.get(i)));
             }
-            return new Optimisation.Result(state, value, solution);
+
+            return this.withSolution(solution);
         }
 
         public Optimisation.State getState() {
@@ -491,6 +581,17 @@ public interface Optimisation {
             return this;
         }
 
+        public Result multipliers(final ConstraintsMap constraintsMap, final Access1D<?> multipliers) {
+            myConstraintsMap = constraintsMap;
+            myMultipliers = multipliers;
+            return this;
+        }
+
+        public Result multipliers(final double... multipliers) {
+            return this.multipliers(ArrayR064.wrap(multipliers));
+        }
+
+        @Override
         public int size() {
             return (int) this.count();
         }
@@ -504,8 +605,64 @@ public interface Optimisation {
             return myState + " " + myValue + " @ " + Access1D.toString(mySolution);
         }
 
+        public Result withNegatedValue() {
+            return this.withValue(-myValue);
+        }
+
+        public Result withSolution(final Access1D<?> solution) {
+
+            Result retVal = new Result(myState, myValue, solution);
+
+            this.multipliers(retVal);
+
+            return retVal;
+        }
+
+        public Result withSolutionLength(final int length) {
+
+            ArrayR064 solution = ArrayR064.make(length);
+
+            for (int i = 0, limit = Math.min(solution.size(), mySolution.size()); i < limit; i++) {
+                solution.set(i, mySolution.doubleValue(i));
+            }
+
+            return this.withSolution(solution);
+        }
+
         public Result withState(final State state) {
-            return new Result(state, myValue, mySolution);
+
+            Result retVal = new Result(state, myValue, mySolution);
+
+            this.multipliers(retVal);
+
+            return retVal;
+        }
+
+        public Result withValue(final double value) {
+
+            Result retVal = new Result(myState, value, mySolution);
+
+            this.multipliers(retVal);
+
+            return retVal;
+        }
+
+        private void multipliers(final Result target) {
+
+            Optional<Access1D<?>> multipliers = this.getMultipliers();
+            if (multipliers.isPresent()) {
+                target.multipliers(multipliers.get());
+            }
+
+            List<KeyedPrimitive<EntryPair<ModelEntity<?>, ConstraintType>>> matchedMultipliers = this.getMatchedMultipliers();
+            if (matchedMultipliers.size() > 0) {
+                target.multipliers(matchedMultipliers);
+            }
+        }
+
+        Result multipliers(final List<KeyedPrimitive<EntryPair<ModelEntity<?>, ConstraintType>>> matchedMultipliers) {
+            myMatchedMultipliers = matchedMultipliers;
+            return this;
         }
 
     }
