@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,29 +40,23 @@ public final class GomorySolver extends GenericSolver {
 
     public static final class ModelIntegration extends ExpressionsBasedModel.Integration<GomorySolver> {
 
+        @Override
         public GomorySolver build(final ExpressionsBasedModel model) {
-            return new GomorySolver(model);
+
+            GomorySolver solver = new GomorySolver(model);
+
+            if (model.options.validate) {
+                solver.setValidator(this.newValidator(model));
+            }
+
+            return solver;
         }
 
+        @Override
         public boolean isCapable(final ExpressionsBasedModel model) {
             return model.isAnyVariableInteger() && !model.isAnyConstraintQuadratic();
         }
 
-        @Override
-        public Result toModelState(final Result solverState, final ExpressionsBasedModel model) {
-            return solverState;
-        }
-
-        @Override
-        public Result toSolverState(final Result modelState, final ExpressionsBasedModel model) {
-            return modelState;
-        }
-
-        @Override
-        protected boolean isSolutionMapped() {
-            return false;
-        }
-
     }
 
     public static final ExpressionsBasedModel.Integration<GomorySolver> INTEGRATION = new GomorySolver.ModelIntegration();
@@ -80,6 +74,7 @@ public final class GomorySolver extends GenericSolver {
         myFunction = myIntegerModel.limitObjective(null, null).toFunction();
     }
 
+    @Override
     public Result solve(final Result kickStarter) {
 
         ModelStrategy strategy = IntegerStrategy.DEFAULT.withGMICutConfiguration(GMI_CUT_CONFIGURATION).newModelStrategy(myIntegerModel);
@@ -90,20 +85,20 @@ public final class GomorySolver extends GenericSolver {
         Result retVal = iterativeSolver.solve();
         this.incrementIterationsCount();
         if (this.isLogProgress()) {
-            this.log("Iteration {}: {}", this.countIterations(), retVal);
             this.log();
+            this.log("Iteration {}: {}", this.countIterations(), retVal);
         }
         while (retVal.getState().isFeasible() && !myIntegerModel.validate(retVal)) {
             iterativeSolver.generateCuts(strategy);
             retVal = iterativeSolver.solve();
             this.incrementIterationsCount();
             if (this.isLogProgress()) {
-                this.log("Iteration {}: {}", this.countIterations(), retVal);
                 this.log();
+                this.log("Iteration {}: {}", this.countIterations(), retVal);
             }
         }
 
-        return retVal;
+        return retVal.withValue(this.evaluateFunction(retVal));
     }
 
     protected Optimisation.Result buildResult() {
