@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@
  */
 package org.ojalgo.optimisation.integer;
 
-import static org.ojalgo.function.constant.PrimitiveMath.*;
+import static org.ojalgo.function.constant.PrimitiveMath.ZERO;
 
 import java.math.BigDecimal;
 import java.util.Arrays;
@@ -50,29 +50,23 @@ public final class IntegerSolver extends GenericSolver {
 
     public static final class ModelIntegration extends ExpressionsBasedModel.Integration<IntegerSolver> {
 
+        @Override
         public IntegerSolver build(final ExpressionsBasedModel model) {
-            return IntegerSolver.make(model);
+
+            IntegerSolver solver = IntegerSolver.make(model);
+
+            if (model.options.validate) {
+                solver.setValidator(this.newValidator(model));
+            }
+
+            return solver;
         }
 
+        @Override
         public boolean isCapable(final ExpressionsBasedModel model) {
             return !model.isAnyConstraintQuadratic();
         }
 
-        @Override
-        public Result toModelState(final Result solverState, final ExpressionsBasedModel model) {
-            return solverState;
-        }
-
-        @Override
-        public Result toSolverState(final Result modelState, final ExpressionsBasedModel model) {
-            return modelState;
-        }
-
-        @Override
-        protected boolean isSolutionMapped() {
-            return false;
-        }
-
     }
 
     /**
@@ -191,6 +185,7 @@ public final class IntegerSolver extends GenericSolver {
         myMinimisation = myIntegerModel.getOptimisationSense() == Optimisation.Sense.MIN;
     }
 
+    @Override
     public Result solve(final Result kickStarter) {
 
         Result point = kickStarter != null ? kickStarter : myIntegerModel.getVariableValues();
@@ -265,13 +260,14 @@ public final class IntegerSolver extends GenericSolver {
         if (bestSolutionFound.getState().isFeasible()) {
             if (solverNormalExit.get()) {
                 return bestSolutionFound.withState(State.OPTIMAL);
+            } else {
+                return bestSolutionFound.withState(State.FEASIBLE);
             }
-            return bestSolutionFound.withState(State.FEASIBLE);
-        }
-        if (solverNormalExit.get()) {
+        } else if (solverNormalExit.get()) {
             return bestSolutionFound.withState(State.INFEASIBLE);
+        } else {
+            return bestSolutionFound.withState(State.FAILED);
         }
-        return bestSolutionFound.withState(State.FAILED);
     }
 
     @Override
@@ -377,19 +373,12 @@ public final class IntegerSolver extends GenericSolver {
 
         strategy.markInteger(key, result);
 
-        double bestIntegerSolutionValue = myBestResultSoFar.getValue();
+        BigDecimal bestIntegerSolutionValue = BigDecimal.valueOf(myBestResultSoFar.getValue());
 
-        if (!strategy.getGapTolerance().isZero(bestIntegerSolutionValue)) {
-
-            double nudge = Math.abs(bestIntegerSolutionValue * strategy.getGapTolerance().epsilon());
-
-            if ((myIntegerModel.getOptimisationSense() != Optimisation.Sense.MAX)) {
-                BigDecimal upper = TypeUtils.toBigDecimal(bestIntegerSolutionValue - nudge, options.feasibility);
-                myIntegerModel.limitObjective(null, upper);
-            } else {
-                BigDecimal lower = TypeUtils.toBigDecimal(bestIntegerSolutionValue + nudge, options.feasibility);
-                myIntegerModel.limitObjective(lower, null);
-            }
+        if (myIntegerModel.getOptimisationSense() != Optimisation.Sense.MAX) {
+            myIntegerModel.limitObjective(null, bestIntegerSolutionValue);
+        } else {
+            myIntegerModel.limitObjective(bestIntegerSolutionValue, null);
         }
     }
 
@@ -585,7 +574,7 @@ public final class IntegerSolver extends GenericSolver {
             displacement = nodeKey.getMinimumDisplacement(i, nodeResult.doubleValue(globalIndex));
             // [0, 0.5]
 
-            if (!options.feasibility.isZero(displacement)) {
+            if (!strategy.getIntegralityTolerance().isZero(displacement)) {
                 // This variable not integer
 
                 comparableDisplacement = strategy.toComparable(i, displacement, myBestResultSoFar != null);
