@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -335,11 +335,6 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
         return new Quaternion(myScalar + arg, i, j, k);
     }
 
-    @Override
-    public Quaternion add(final float scalarAddend) {
-        return this.add((double) scalarAddend);
-    }
-
     @Override
     public Quaternion add(final Quaternion arg) {
 
@@ -435,11 +430,6 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
         return new Quaternion(tmpScalar, tmpI, tmpJ, tmpK);
     }
 
-    @Override
-    public Quaternion divide(final float scalarDivisor) {
-        return this.divide((double) scalarDivisor);
-    }
-
     /**
      * Will calculate <code>this * reciprocal(arg)</code> which is <b>not</b> the same as
      * <code>reciprocal(arg) * this</code>.
@@ -458,8 +448,8 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
     }
 
     @Override
-    public double doubleValue(final long index) {
-        switch ((int) index) {
+    public double doubleValue(final int index) {
+        switch (index) {
         case 0:
             return myScalar;
         case 1:
@@ -498,11 +488,12 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
     }
 
     @Override
-    public double doubleValue(final long row, final long col) {
+    public double doubleValue(final int row, final int col) {
         if (row == col) {
             return myScalar;
+        } else {
+            return this.doubleValue(row + col * 4);
         }
-        return this.doubleValue(row + col * 4L);
     }
 
     @Override
@@ -544,12 +535,12 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
 
     @Override
     public Double get(final long index) {
-        return this.doubleValue(index);
+        return Double.valueOf(this.doubleValue(index));
     }
 
     @Override
     public Double get(final long row, final long col) {
-        return this.doubleValue(row, col);
+        return Double.valueOf(this.doubleValue(row, col));
     }
 
     /**
@@ -660,11 +651,6 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
         return new Quaternion(tmpScalar, tmpI, tmpJ, tmpK);
     }
 
-    @Override
-    public Quaternion multiply(final float scalarMultiplicand) {
-        return this.multiply((double) scalarMultiplicand);
-    }
-
     @Override
     public Quaternion multiply(final Quaternion arg) {
 
@@ -727,11 +713,6 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
         return new Quaternion(myScalar - arg, i, j, k);
     }
 
-    @Override
-    public Quaternion subtract(final float scalarSubtrahend) {
-        return this.subtract((double) scalarSubtrahend);
-    }
-
     @Override
     public Quaternion subtract(final Quaternion arg) {
 
@@ -770,7 +751,7 @@ public final class Quaternion implements SelfDeclaringScalar<Quaternion>, Access
 
     public MatrixStore<ComplexNumber> toComplexMatrix() {
 
-        final GenericStore<ComplexNumber> retVal = GenericStore.COMPLEX.make(2L, 2L);
+        final GenericStore<ComplexNumber> retVal = GenericStore.C128.make(2L, 2L);
 
         retVal.set(0L, ComplexNumber.of(myScalar, i));
         retVal.set(1L, ComplexNumber.of(-j, k));
