@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,6 @@
 package org.ojalgo.structure;
 
 import java.util.List;
-import java.util.concurrent.atomic.DoubleAdder;
 
 import org.ojalgo.function.VoidFunction;
 import org.ojalgo.function.aggregator.Aggregator;
@@ -91,39 +90,48 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
             this(values, -1L, values.count() - 1L);
         }
 
+        @Override
         public double doubleValue() {
             return myValues.doubleValue(myCursor);
         }
 
+        @Override
         public long estimateSize() {
             return myLastCursor - myCursor;
         }
 
+        @Override
         public N get() {
             return myValues.get(myCursor);
         }
 
+        @Override
         public boolean hasNext() {
             return myCursor < myLastCursor;
         }
 
+        @Override
         public boolean hasPrevious() {
             return myCursor > 0;
         }
 
+        @Override
         public long index() {
             return myCursor;
         }
 
+        @Override
         public ElementView<N> iterator() {
             return new ElementView<>(myValues);
         }
 
+        @Override
         public ElementView<N> next() {
             myCursor++;
             return this;
         }
 
+        @Override
         public ElementView<N> previous() {
             myCursor--;
             return this;
@@ -134,6 +142,7 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
             return myCursor + " = " + myValues.get(myCursor);
         }
 
+        @Override
         public ElementView<N> trySplit() {
 
             long remaining = myLastCursor - myCursor;
@@ -167,18 +176,22 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
             mySelection = selection;
         }
 
+        @Override
         public long count() {
             return mySelection.length;
         }
 
-        public double doubleValue(final long index) {
-            return myFullData.doubleValue(mySelection[Math.toIntExact(index)]);
+        @Override
+        public double doubleValue(final int index) {
+            return myFullData.doubleValue(mySelection[index]);
         }
 
+        @Override
         public N get(final long index) {
             return myFullData.get(mySelection[Math.toIntExact(index)]);
         }
 
+        @Override
         public void supplyTo(final Mutate1D receiver) {
             for (int i = 0, limit = mySelection.length; i < limit; i++) {
                 receiver.set(i, myFullData.get(mySelection[i]));
@@ -207,7 +220,9 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
         void visitOne(long index, VoidFunction<N> visitor);
 
         default void visitRange(final long first, final long limit, final VoidFunction<N> visitor) {
-            Structure1D.loopRange(first, limit, i -> this.visitOne(i, visitor));
+            for (long i = first; i < limit; i++) {
+                this.visitOne(i, visitor);
+            }
         }
 
     }
@@ -219,12 +234,12 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
                 return access.count();
             }
 
-            public double doubleValue(final long index) {
+            public double doubleValue(final int index) {
                 return access.doubleValue(index);
             }
 
             public Double get(final long index) {
-                return access.doubleValue(index);
+                return Double.valueOf(access.doubleValue(index));
             }
 
             @Override
@@ -236,7 +251,7 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
     }
 
     /**
-     * Tests if the two data strauctures are numerically equal to the given accuracy. (Only works with real
+     * Tests if the two data structures are numerically equal to the given accuracy. (Only works with real
      * numbers, and can't handle more than "double precision".) You have to implement your own version to
      * handle other cases.
      */
@@ -300,8 +315,8 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
                 return target.length;
             }
 
-            public double doubleValue(final long index) {
-                return target[Math.toIntExact(index)];
+            public double doubleValue(final int index) {
+                return target[index];
             }
 
             public Double get(final long index) {
@@ -323,12 +338,12 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
                 return target.size();
             }
 
-            public double doubleValue(final long index) {
-                return NumberDefinition.doubleValue(target.get((int) index));
+            public double doubleValue(final int index) {
+                return NumberDefinition.doubleValue(target.get(index));
             }
 
             public N get(final long index) {
-                return target.get((int) index);
+                return target.get(Math.toIntExact(index));
             }
 
             @Override
@@ -346,12 +361,12 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
                 return target.length;
             }
 
-            public double doubleValue(final long index) {
-                return NumberDefinition.doubleValue(target[(int) index]);
+            public double doubleValue(final int index) {
+                return NumberDefinition.doubleValue(target[index]);
             }
 
             public N get(final long index) {
-                return target[(int) index];
+                return target[Math.toIntExact(index)];
             }
 
             @Override
@@ -382,6 +397,10 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
         };
     }
 
+    default <K> Keyed1D<K, N> asKeyed1D(final IndexMapper<K> indexMapper) {
+        return new Keyed1D<>(this, indexMapper);
+    }
+
     /**
      * Will calculate y = y + a x, will add "a" times "this" to "y"
      *
@@ -389,11 +408,17 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
      * @param y The "vector" to update
      */
     default void axpy(final double a, final Mutate1D.Modifiable<?> y) {
-        Structure1D.loopMatching(this, y, i -> y.add(i, a * this.doubleValue(i)));
+        for (ElementView1D<N, ?> element : this.nonzeros()) {
+            y.add(element.index(), a * element.doubleValue());
+        }
+    }
+
+    default byte byteValue(final int index) {
+        return (byte) this.shortValue(index);
     }
 
     default byte byteValue(final long index) {
-        return (byte) this.shortValue(index);
+        return this.byteValue(Math.toIntExact(index));
     }
 
     /**
@@ -403,12 +428,21 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
      * @return The dot product
      */
     default double dot(final Access1D<?> vector) {
-        DoubleAdder retVal = new DoubleAdder();
-        Structure1D.loopMatching(this, vector, i -> retVal.add(this.doubleValue(i) * vector.doubleValue(i)));
-        return retVal.doubleValue();
+
+        double retVal = 0D;
+
+        for (ElementView1D<N, ?> element : this.nonzeros()) {
+            retVal += element.doubleValue() * vector.doubleValue(element.index());
+        }
+
+        return retVal;
     }
 
-    double doubleValue(long index);
+    double doubleValue(int index);
+
+    default double doubleValue(final long index) {
+        return this.doubleValue(Math.toIntExact(index));
+    }
 
     /**
      * Returns an Iterable of ElementView1D. It allows to iterate over the instance's element "positions"
@@ -418,20 +452,38 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
         return new Access1D.ElementView<>(this);
     }
 
-    default float floatValue(final long index) {
+    default float floatValue(final int index) {
         return (float) this.doubleValue(index);
     }
 
+    default float floatValue(final long index) {
+        return this.floatValue(Math.toIntExact(index));
+    }
+
     N get(long index);
 
-    default int intValue(final long index) {
+    default int intValue(final int index) {
         return (int) this.longValue(index);
     }
 
-    default long longValue(final long index) {
+    default int intValue(final long index) {
+        return this.intValue(Math.toIntExact(index));
+    }
+
+    default long longValue(final int index) {
         return Math.round(this.doubleValue(index));
     }
 
+    default long longValue(final long index) {
+        return this.longValue(Math.toIntExact(index));
+    }
+
+    /**
+     * Similar to {@link #elements()} but avoids elements that are structurally known to be zero. (That does
+     * not eliminate all zero-values from this view.) With an arbitrary (dense) unstructured implementation
+     * the {@link #nonzeros()} and {@link #elements()} methods do the same thing! Only some specific
+     * implementations are able to actually exploit structure/sparsity to view fewer elements.
+     */
     default ElementView1D<N, ?> nonzeros() {
         return this.elements();
     }
@@ -443,10 +495,14 @@ public interface Access1D<N extends Comparable<N>> extends Structure1D {
         return new Access1D.SelectionView<>(this, selection);
     }
 
-    default short shortValue(final long index) {
+    default short shortValue(final int index) {
         return (short) this.intValue(index);
     }
 
+    default short shortValue(final long index) {
+        return this.shortValue(Math.toIntExact(index));
+    }
+
     default void supplyTo(final double[] receiver) {
         int limit = Math.min(receiver.length, (int) this.count());
         for (int i = 0; i < limit; i++) {
