@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,7 @@ import java.util.List;
 import org.ojalgo.function.FunctionSet;
 import org.ojalgo.function.NullaryFunction;
 import org.ojalgo.scalar.Scalar.Factory;
+import org.ojalgo.type.math.MathType;
 
 public interface Factory2D<I extends Structure2D> extends FactorySupplement {
 
@@ -108,6 +109,10 @@ public interface Factory2D<I extends Structure2D> extends FactorySupplement {
                 return Factory2D.this.scalar();
             }
 
+            public MathType getMathType() {
+                return Factory2D.this.getMathType();
+            }
+
         };
     }
 
