@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,14 @@ package org.ojalgo.structure;
 
 import org.ojalgo.function.FunctionSet;
 import org.ojalgo.scalar.Scalar;
+import org.ojalgo.type.math.MathType;
 
 interface FactorySupplement {
 
     FunctionSet<?> function();
 
+    MathType getMathType();
+
     Scalar.Factory<?> scalar();
 
 }
