@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -55,15 +55,21 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void fillColumn(final long row, final long col, final Access1D<N> values) {
-            this.loopColumn(row, col, (r, c) -> this.fillOne(r, c, values.get(r - row)));
+            for (long i = row, limit = this.countRows(); i < limit; i++) {
+                this.fillOne(i, col, values.get(i - row));
+            }
         }
 
         default void fillColumn(final long row, final long col, final N value) {
-            this.loopColumn(row, col, (r, c) -> this.fillOne(r, c, value));
+            for (long i = row, limit = this.countRows(); i < limit; i++) {
+                this.fillOne(i, col, value);
+            }
         }
 
         default void fillColumn(final long row, final long col, final NullaryFunction<?> supplier) {
-            this.loopColumn(row, col, (r, c) -> this.fillOne(r, c, supplier));
+            for (long i = row, limit = this.countRows(); i < limit; i++) {
+                this.fillOne(i, col, supplier);
+            }
         }
 
         default void fillColumn(final long col, final N value) {
@@ -79,15 +85,21 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void fillDiagonal(final long row, final long col, final Access1D<N> values) {
-            this.loopDiagonal(row, col, (r, c) -> this.fillOne(r, c, values.get(r - row)));
+            for (long ij = 0L, limit = Math.min(this.countRows() - row, this.countColumns() - col); ij < limit; ij++) {
+                this.fillOne(row + ij, col + ij, values.get(ij));
+            }
         }
 
         default void fillDiagonal(final long row, final long col, final N value) {
-            this.loopDiagonal(row, col, (r, c) -> this.fillOne(r, c, value));
+            for (long ij = 0L, limit = Math.min(this.countRows() - row, this.countColumns() - col); ij < limit; ij++) {
+                this.fillOne(row + ij, col + ij, value);
+            }
         }
 
         default void fillDiagonal(final long row, final long col, final NullaryFunction<?> supplier) {
-            this.loopDiagonal(row, col, (r, c) -> this.fillOne(r, c, supplier));
+            for (long ij = 0L, limit = Math.min(this.countRows() - row, this.countColumns() - col); ij < limit; ij++) {
+                this.fillOne(row + ij, col + ij, supplier);
+            }
         }
 
         default void fillDiagonal(final N value) {
@@ -157,15 +169,21 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void fillRow(final long row, final long col, final Access1D<N> values) {
-            this.loopRow(row, col, (r, c) -> this.fillOne(r, c, values.get(c - col)));
+            for (long j = col, limit = this.countColumns(); j < limit; j++) {
+                this.fillOne(row, j, values.get(j - col));
+            }
         }
 
         default void fillRow(final long row, final long col, final N value) {
-            this.loopRow(row, col, (r, c) -> this.fillOne(r, c, value));
+            for (long j = col, limit = this.countColumns(); j < limit; j++) {
+                this.fillOne(row, j, value);
+            }
         }
 
         default void fillRow(final long row, final long col, final NullaryFunction<?> supplier) {
-            this.loopRow(row, col, (r, c) -> this.fillOne(r, c, supplier));
+            for (long j = col, limit = this.countColumns(); j < limit; j++) {
+                this.fillOne(row, j, supplier);
+            }
         }
 
         default void fillRow(final long row, final N value) {
@@ -266,7 +284,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void modifyColumn(final long row, final long col, final UnaryFunction<N> modifier) {
-            this.loopColumn(row, col, (r, c) -> this.modifyOne(r, c, modifier));
+            for (long i = row, limit = this.countRows(); i < limit; i++) {
+                this.modifyOne(i, col, modifier);
+            }
         }
 
         default void modifyColumn(final long col, final UnaryFunction<N> modifier) {
@@ -274,7 +294,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void modifyDiagonal(final long row, final long col, final UnaryFunction<N> modifier) {
-            this.loopDiagonal(row, col, (r, c) -> this.modifyOne(r, c, modifier));
+            for (long ij = 0L, limit = Math.min(this.countRows() - row, this.countColumns() - col); ij < limit; ij++) {
+                this.modifyOne(row + ij, col + ij, modifier);
+            }
         }
 
         default void modifyDiagonal(final UnaryFunction<N> modifier) {
@@ -282,19 +304,27 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void modifyMatchingInColumns(final Access1D<N> left, final BinaryFunction<N> function) {
-            left.loopAll(r -> this.modifyRow(r, function.first(left.get(r))));
+            for (long r = 0L; r < left.count(); r++) {
+                this.modifyRow(r, function.first(left.get(r)));
+            }
         }
 
         default void modifyMatchingInColumns(final BinaryFunction<N> function, final Access1D<N> right) {
-            right.loopAll(r -> this.modifyRow(r, function.second(right.get(r))));
+            for (long r = 0L; r < right.count(); r++) {
+                this.modifyRow(r, function.second(right.get(r)));
+            }
         }
 
         default void modifyMatchingInRows(final Access1D<N> left, final BinaryFunction<N> function) {
-            left.loopAll(c -> this.modifyColumn(c, function.first(left.get(c))));
+            for (long c = 0L; c < left.count(); c++) {
+                this.modifyColumn(c, function.first(left.get(c)));
+            }
         }
 
         default void modifyMatchingInRows(final BinaryFunction<N> function, final Access1D<N> right) {
-            right.loopAll(c -> this.modifyColumn(c, function.second(right.get(c))));
+            for (long c = 0L; c < right.count(); c++) {
+                this.modifyColumn(c, function.second(right.get(c)));
+            }
         }
 
         void modifyOne(long row, long col, UnaryFunction<N> modifier);
@@ -306,7 +336,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
         }
 
         default void modifyRow(final long row, final long col, final UnaryFunction<N> modifier) {
-            this.loopRow(row, col, (r, c) -> this.modifyOne(r, c, modifier));
+            for (long j = col, limit = this.countColumns(); j < limit; j++) {
+                this.modifyOne(row, j, modifier);
+            }
         }
 
         default void modifyRow(final long row, final UnaryFunction<N> modifier) {
@@ -316,9 +348,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
     }
 
     /**
-     * Apart from extending {@link Receiver} this interface extends {@link Modifiable} and
-     * {@link Exchangeable} which both imply access to existing elements as well as {@link Access2D} that
-     * dictates explicit access.
+     * Apart from extending {@link org.ojalgo.structure.Mutate2D.Receiver} this interface extends
+     * {@link org.ojalgo.structure.Mutate2D.Modifiable} and {@link Exchangeable} which both imply access to
+     * existing elements as well as {@link Access2D} that dictates explicit access.
      *
      * @author apete
      */
@@ -332,53 +364,120 @@ public interface Mutate2D extends Structure2D, Mutate1D {
 
         @Override
         default void accept(final Access2D<?> supplied) {
+
             if (this.isAcceptable(supplied)) {
-                supplied.loopAll((r, c) -> this.set(r, c, supplied.get(r, c)));
+
+                long limitRows = Math.min(this.countRows(), supplied.countRows());
+                long limitCols = Math.min(this.countColumns(), supplied.countColumns());
+                for (long j = 0L; j < limitCols; j++) {
+                    for (long i = 0L; i < limitRows; i++) {
+                        this.set(i, j, supplied.get(i, j));
+                    }
+                }
+
             } else {
                 throw new ProgrammingError("Not acceptable!");
             }
         }
 
         default boolean isAcceptable(final Structure2D supplier) {
-            return (this.countRows() >= supplier.countRows()) && (this.countColumns() >= supplier.countColumns());
+            return this.countRows() >= supplier.countRows() && this.countColumns() >= supplier.countColumns();
         }
 
     }
 
     @Override
-    default void set(final long index, final byte addend) {
+    default void set(final int index, final byte value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
     @Override
-    default void set(final long index, final Comparable<?> addend) {
+    default void set(final int index, final double value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
     @Override
-    default void set(final long index, final double addend) {
+    default void set(final int index, final float value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
     @Override
-    default void set(final long index, final float addend) {
+    default void set(final int index, final int value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    default void set(final int row, final int col, final byte value) {
+        this.set(row, col, (short) value);
+    }
+
+    void set(int row, int col, double value);
+
+    default void set(final int row, final int col, final float value) {
+        this.set(row, col, (double) value);
+    }
+
+    default void set(final int row, final int col, final int value) {
+        this.set(row, col, (long) value);
+    }
+
+    default void set(final int row, final int col, final long value) {
+        this.set(row, col, (double) value);
+    }
+
+    default void set(final int row, final int col, final short value) {
+        this.set(row, col, (int) value);
     }
 
     @Override
-    default void set(final long index, final int addend) {
+    default void set(final int index, final long value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
     @Override
-    default void set(final long index, final long addend) {
+    default void set(final int index, final short value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final byte value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final Comparable<?> value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final double value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final float value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final int value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
+    }
+
+    @Override
+    default void set(final long index, final long value) {
+        long structure = this.countRows();
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
     default void set(final long row, final long col, final byte value) {
@@ -387,7 +486,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
 
     void set(long row, long col, Comparable<?> value);
 
-    void set(long row, long col, double value);
+    default void set(final long row, final long col, final double value) {
+        this.set(Math.toIntExact(row), Math.toIntExact(col), value);
+    }
 
     default void set(final long row, final long col, final float value) {
         this.set(row, col, (double) value);
@@ -406,9 +507,9 @@ public interface Mutate2D extends Structure2D, Mutate1D {
     }
 
     @Override
-    default void set(final long index, final short addend) {
+    default void set(final long index, final short value) {
         long structure = this.countRows();
-        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), addend);
+        this.set(Structure2D.row(index, structure), Structure2D.column(index, structure), value);
     }
 
 }
