@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -361,6 +361,10 @@ public interface Structure2D extends Structure1D {
 
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     @FunctionalInterface
     public interface RowColumnCallback {
 
@@ -383,8 +387,8 @@ public interface Structure2D extends Structure1D {
 
         public RowColumnKey(final R theRow, final C theCol) {
             super();
-            this.row = theRow;
-            this.column = theCol;
+            row = theRow;
+            column = theCol;
         }
 
         @SuppressWarnings("rawtypes")
@@ -437,15 +441,19 @@ public interface Structure2D extends Structure1D {
             myColumnMapper = columnMapper;
         }
 
+        public long toColumnIndex(final C columnKey) {
+            return myColumnMapper.toIndex(columnKey);
+        }
+
         public C toColumnKey(final long index) {
-            final long col = Structure2D.column(index, myStructure);
+            long col = Structure2D.column(index, myStructure);
             return myColumnMapper.toKey(col);
         }
 
         public long toIndex(final R rowKey, final C colKey) {
 
-            final long row = myRowMapper.toIndex(rowKey);
-            final long col = myColumnMapper.toIndex(colKey);
+            long row = myRowMapper.toIndex(rowKey);
+            long col = myColumnMapper.toIndex(colKey);
 
             return Structure2D.index(myStructure, row, col);
         }
@@ -458,8 +466,12 @@ public interface Structure2D extends Structure1D {
             return RowColumnKey.of(this.toRowKey(index), this.toColumnKey(index));
         }
 
+        public long toRowIndex(final R rowKey) {
+            return myRowMapper.toIndex(rowKey);
+        }
+
         public R toRowKey(final long index) {
-            final long row = Structure2D.row(index, myStructure);
+            long row = Structure2D.row(index, myStructure);
             return myRowMapper.toKey(row);
         }
 
@@ -525,9 +537,13 @@ public interface Structure2D extends Structure1D {
         return structure instanceof Structure2D ? Math.min(((Structure2D) structure).limitOfRow((int) row), defaultAndMaximum) : defaultAndMaximum;
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     static void loopMatching(final Structure2D structureA, final Structure2D structureB, final RowColumnCallback callback) {
-        final long tmpCountRows = Math.min(structureA.countRows(), structureB.countRows());
-        final long tmpCountColumns = Math.min(structureA.countColumns(), structureB.countColumns());
+        long tmpCountRows = Math.min(structureA.countRows(), structureB.countRows());
+        long tmpCountColumns = Math.min(structureA.countColumns(), structureB.countColumns());
         for (long j = 0L; j < tmpCountColumns; j++) {
             for (long i = 0L; i < tmpCountRows; i++) {
                 callback.call(i, j);
@@ -703,6 +719,10 @@ public interface Structure2D extends Structure1D {
         return this.getColDim();
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopAll(final RowColumnCallback callback) {
         final long tmpCountRows = this.countRows();
         final long tmpCountColumns = this.countColumns();
@@ -713,32 +733,50 @@ public interface Structure2D extends Structure1D {
         }
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopColumn(final long row, final long col, final RowColumnCallback callback) {
-        final long tmpCountRows = this.countRows();
-        for (long i = row; i < tmpCountRows; i++) {
+        for (long i = row, limit = this.countRows(); i < limit; i++) {
             callback.call(i, col);
         }
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopColumn(final long col, final RowColumnCallback callback) {
         this.loopColumn(0L, col, callback);
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopDiagonal(final long row, final long col, final RowColumnCallback callback) {
-        final long tmpLimit = Math.min(this.countRows() - row, this.countColumns() - col);
-        for (long ij = 0L; ij < tmpLimit; ij++) {
+        for (long ij = 0L, limit = Math.min(this.countRows() - row, this.countColumns() - col); ij < limit; ij++) {
             callback.call(row + ij, col + ij);
         }
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopRow(final long row, final long col, final RowColumnCallback callback) {
-        final long tmpCountColumns = this.countColumns();
-        for (long j = col; j < tmpCountColumns; j++) {
+        for (long j = col, limit = this.countColumns(); j < limit; j++) {
             callback.call(row, j);
         }
     }
 
+    /**
+     * @deprecated v53 Will be removed!
+     */
+    @Deprecated
     default void loopRow(final long row, final RowColumnCallback callback) {
         this.loopRow(row, 0L, callback);
     }
+
 }
