@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,7 @@ import org.ojalgo.array.DenseArray;
 import org.ojalgo.structure.Access2D;
 import org.ojalgo.structure.Factory2D;
 import org.ojalgo.structure.Mutate2D;
+import org.ojalgo.type.math.MathType;
 
 public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTensor<N, MatrixTensor<N>> implements Access2D<N>, Mutate2D.Receiver<N> {
 
@@ -59,6 +60,11 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
             return true;
         }
 
+        @Override
+        public MathType getMathType() {
+            return myFactory.getMathType();
+        }
+
         @Override
         public int hashCode() {
             final int prime = 31;
@@ -66,6 +72,7 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
             return prime * result + (myFactory == null ? 0 : myFactory.hashCode());
         }
 
+        @Override
         public MatrixTensor<N> make(final long rows, final long columns) {
             if (rows != columns) {
                 throw new IllegalArgumentException();
@@ -90,6 +97,7 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         myArray = factory.make(dimensions, dimensions);
     }
 
+    @Override
     public MatrixTensor<N> add(final MatrixTensor<N> addend) {
 
         MatrixTensor<N> retVal = this.newSameShape();
@@ -99,10 +107,17 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return retVal;
     }
 
+    @Override
+    public byte byteValue(final int row, final int col) {
+        return myArray.byteValue(row, col);
+    }
+
+    @Override
     public byte byteValue(final long row, final long col) {
         return myArray.byteValue(row, col);
     }
 
+    @Override
     public MatrixTensor<N> conjugate() {
 
         MatrixTensor<N> retVal = this.newSameShape();
@@ -117,18 +132,27 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return retVal;
     }
 
+    @Override
     public long count() {
         return myArray.count();
     }
 
+    @Override
     public long countColumns() {
         return myArray.countColumns();
     }
 
+    @Override
     public long countRows() {
         return myArray.countRows();
     }
 
+    @Override
+    public double doubleValue(final int row, final int col) {
+        return myArray.doubleValue(row, col);
+    }
+
+    @Override
     public double doubleValue(final long row, final long col) {
         return myArray.doubleValue(row, col);
     }
@@ -159,10 +183,17 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return true;
     }
 
+    @Override
+    public float floatValue(final int row, final int col) {
+        return myArray.floatValue(row, col);
+    }
+
+    @Override
     public float floatValue(final long row, final long col) {
         return myArray.floatValue(row, col);
     }
 
+    @Override
     public N get(final long row, final long col) {
         return myArray.get(row, col);
     }
@@ -175,14 +206,27 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return prime * result + (myFactory == null ? 0 : myFactory.hashCode());
     }
 
+    @Override
+    public int intValue(final int row, final int col) {
+        return myArray.intValue(row, col);
+    }
+
+    @Override
     public int intValue(final long row, final long col) {
         return myArray.intValue(row, col);
     }
 
+    @Override
+    public long longValue(final int row, final int col) {
+        return myArray.longValue(row, col);
+    }
+
+    @Override
     public long longValue(final long row, final long col) {
         return myArray.longValue(row, col);
     }
 
+    @Override
     public MatrixTensor<N> multiply(final double scalarMultiplicand) {
 
         MatrixTensor<N> retVal = this.newSameShape();
@@ -192,6 +236,7 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return retVal;
     }
 
+    @Override
     public MatrixTensor<N> multiply(final N scalarMultiplicand) {
 
         MatrixTensor<N> retVal = this.newSameShape();
@@ -201,6 +246,7 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return retVal;
     }
 
+    @Override
     public MatrixTensor<N> negate() {
 
         MatrixTensor<N> retVal = this.newSameShape();
@@ -210,38 +256,82 @@ public final class MatrixTensor<N extends Comparable<N>> extends ArrayBasedTenso
         return retVal;
     }
 
+    @Override
     public double norm() {
         return this.norm(myArray);
     }
 
+    @Override
+    public void set(final int row, final int col, final byte value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final double value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final float value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final int value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final long value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
+    public void set(final int row, final int col, final short value) {
+        myArray.set(row, col, value);
+    }
+
+    @Override
     public void set(final long row, final long col, final byte value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final Comparable<?> value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final double value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final float value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final int value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final long value) {
         myArray.set(row, col, value);
     }
 
+    @Override
     public void set(final long row, final long col, final short value) {
         myArray.set(row, col, value);
     }
 
+    @Override
+    public short shortValue(final int row, final int col) {
+        return myArray.shortValue(row, col);
+    }
+
+    @Override
     public short shortValue(final long row, final long col) {
         return myArray.shortValue(row, col);
     }
