@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -83,16 +83,31 @@ public interface AutoConsumer<T> extends AutoCloseable, Consumer<T>, AutoFunctio
         return ShardedConsumer.of(distributor, consumers);
     }
 
+    /**
+     * @see #write(Object)
+     */
+    @Override
     default void accept(final T item) {
         this.write(item);
     }
 
+    @Override
     default void close() throws Exception {
         // Default implementation does nothing
     }
 
+    /**
+     * Write the item to the consumer.
+     * 
+     * @param item The item to be written
+     */
     void write(T item);
 
+    /**
+     * Write the batch (collection of items) to the consumer.
+     * 
+     * @param batch The batch to be written
+     */
     default void writeBatch(final Iterable<? extends T> batch) {
         for (T item : batch) {
             this.write(item);
