@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -70,7 +70,7 @@ import org.ojalgo.type.context.NumberContext;
  */
 public abstract class TestUtils /* extends Assertions */ {
 
-    private static final NumberContext EQUALS = NumberContext.of(12, 14);
+    private static final NumberContext EQUALS = NumberContext.of(12);
 
     public static void assertBounds(final Comparable<?> lower, final Access1D<?> values, final Comparable<?> upper, final NumberContext precision) {
         for (ElementView1D<?, ?> tmpValue : values.elements()) {
@@ -89,6 +89,17 @@ public abstract class TestUtils /* extends Assertions */ {
         }
     }
 
+    public static void assertComplexEquals(final Access1D<ComplexNumber> expected, final Access1D<ComplexNumber> actual) {
+        TestUtils.assertComplexEquals(expected, actual, EQUALS);
+    }
+
+    public static void assertComplexEquals(final Access1D<ComplexNumber> expected, final Access1D<ComplexNumber> actual, final NumberContext context) {
+        TestUtils.assertEquals(expected, actual, context);
+        for (int i = 0; i < expected.size(); i++) {
+            TestUtils.assertEquals(expected.get(i), actual.get(i), context);
+        }
+    }
+
     public static void assertEquals(final Access1D<?> expected, final Access1D<?> actual) {
         TestUtils.assertEquals(expected, actual, EQUALS);
     }
@@ -223,6 +234,10 @@ public abstract class TestUtils /* extends Assertions */ {
         }
     }
 
+    public static void assertNull(final Object actual) {
+        Assertions.assertNull(actual);
+    }
+
     public static void assertEquals(final Object expected, final Object actual) {
         Assertions.assertEquals(expected, actual);
     }
@@ -389,6 +404,12 @@ public abstract class TestUtils /* extends Assertions */ {
         }
     }
 
+    public static void assertInRange(final double first, final double limit, final double actual) {
+        if (first > actual || actual >= limit) {
+            TestUtils.fail("Not in range!");
+        }
+    }
+
     public static void assertInRange(final int first, final int limit, final int actual) {
         if (first > actual || actual >= limit) {
             TestUtils.fail("Not in range!");
@@ -431,6 +452,9 @@ public abstract class TestUtils /* extends Assertions */ {
         }
     }
 
+    /**
+     * Will compare state, value, solution and multiplier if available.
+     */
     public static void assertResult(final Optimisation.Result expected, final Optimisation.Result actual) {
         TestUtils.assertResult(expected, actual, EQUALS);
     }
@@ -462,6 +486,12 @@ public abstract class TestUtils /* extends Assertions */ {
         }
     }
 
+    public static void assertSolutionValid(final ExpressionsBasedModel model, final Access1D<BigDecimal> solution, final NumberContext accuracy) {
+        if (!model.validate(solution, accuracy, BasicLogger.ERROR)) {
+            Assertions.fail("Solution not valid!");
+        }
+    }
+
     public static void assertStateAndSolution(final Optimisation.Result expected, final Optimisation.Result actual) {
         TestUtils.assertStateAndSolution(expected, actual, EQUALS);
     }
@@ -541,7 +571,7 @@ public abstract class TestUtils /* extends Assertions */ {
 
     public static PhysicalStore<ComplexNumber> makeRandomComplexStore(final int numberOfRows, final int numberOfColumns) {
 
-        PhysicalStore<ComplexNumber> retVal = GenericStore.COMPLEX.make(numberOfRows, numberOfColumns);
+        PhysicalStore<ComplexNumber> retVal = GenericStore.C128.make(numberOfRows, numberOfColumns);
 
         Uniform tmpArgGen = new Uniform(PrimitiveMath.ZERO, PrimitiveMath.TWO_PI);
 
@@ -581,7 +611,7 @@ public abstract class TestUtils /* extends Assertions */ {
             boolean failed = false;
 
             if (expectedState == actualState) {
-
+                failed = false;
             } else if (expectedState.isDistinct() && !actualState.isDistinct() || expectedState.isOptimal() && !actualState.isOptimal()) {
                 failed = true;
             } else if (expectedState.isFeasible() && !actualState.isFeasible() || expectedState.isApproximate() && !actualState.isApproximate()) {
