@@ -2,10 +2,9 @@ package org.ojalgo.data.domain.finance.portfolio;
 
 import java.math.BigDecimal;
 
-import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 import org.ojalgo.TestUtils;
-import org.ojalgo.matrix.Primitive64Matrix;
+import org.ojalgo.matrix.MatrixR064;
 import org.ojalgo.netio.BasicLogger;
 import org.ojalgo.optimisation.Optimisation.State;
 
@@ -230,13 +229,13 @@ public class GitHubIssue24 extends FinancePortfolioTests {
                         3.9609092637332395E-4, -0.0012151819283578002, 0.00637053301651962, 0.006696955305486993, -1.5727884198208E-4, 0.004083169149099768,
                         0.00288619107186696, -3.18539969399288E-4, 0.006329914612237217, 0.006494430855358576, 0.0064963600000000005 } };
 
-        MarketEquilibrium marketEquilibrium = new MarketEquilibrium(Primitive64Matrix.FACTORY.rows(covariance));
+        MarketEquilibrium marketEquilibrium = new MarketEquilibrium(MatrixR064.FACTORY.rows(covariance));
 
         if (cleanCovariances) {
             marketEquilibrium = marketEquilibrium.clean();
         }
 
-        MarkowitzModel retVal = new MarkowitzModel(marketEquilibrium, Primitive64Matrix.FACTORY.rows(expectedReturns));
+        MarkowitzModel retVal = new MarkowitzModel(marketEquilibrium, MatrixR064.FACTORY.rows(expectedReturns));
 
         retVal.optimiser().debug(debugOptimisationSolver).validate(validateOptimisationModel);
 
@@ -251,7 +250,6 @@ public class GitHubIssue24 extends FinancePortfolioTests {
     }
 
     @Test
-    @Disabled("Google Finance stopped working")
     public void testHanging() throws Exception {
 
         MarkowitzModel markowitzModel = GitHubIssue24.buildMarkowitzModel(2.5E-5, false, false, false);
