@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@ import java.io.IOException;
 import java.math.MathContext;
 import java.nio.file.Files;
 import java.nio.file.Paths;
+import java.util.Comparator;
 import java.util.List;
 import java.util.Optional;
 import java.util.stream.Collectors;
@@ -37,6 +38,7 @@ import org.ojalgo.RecoverableCondition;
 import org.ojalgo.TestUtils;
 import org.ojalgo.array.Array1D;
 import org.ojalgo.array.Array2D;
+import org.ojalgo.array.ArrayR064;
 import org.ojalgo.function.aggregator.Aggregator;
 import org.ojalgo.function.constant.PrimitiveMath;
 import org.ojalgo.matrix.P20050125Case;
@@ -48,8 +50,10 @@ import org.ojalgo.matrix.store.GenericStore;
 import org.ojalgo.matrix.store.MatrixStore;
 import org.ojalgo.matrix.store.PhysicalStore;
 import org.ojalgo.matrix.store.Primitive64Store;
+import org.ojalgo.matrix.store.RawStore;
 import org.ojalgo.matrix.task.SolverTask;
 import org.ojalgo.netio.BasicLogger;
+import org.ojalgo.random.Uniform;
 import org.ojalgo.scalar.ComplexNumber;
 import org.ojalgo.structure.Access1D;
 import org.ojalgo.type.context.NumberContext;
@@ -67,6 +71,62 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
 
     }
 
+    /**
+     * Creates a square symmetric random matrix with the specified eigenvalues.
+     */
+    public static MatrixStore<Double> newRandom(final double... eigenvalues) {
+
+        int dim = eigenvalues.length;
+
+        Primitive64Store seed = Primitive64Store.FACTORY.makeFilled(dim, dim, Uniform.standard());
+        QR<Double> qr = QR.R064.make(seed);
+        qr.decompose(seed);
+
+        MatrixStore<Double> mtrxQ = qr.getQ();
+
+        DiagonalStore<Double, ArrayR064> mtrxD = Primitive64Store.FACTORY.makeDiagonal(ArrayR064.wrap(eigenvalues)).get();
+
+        return mtrxQ.multiply(mtrxD).multiply(mtrxQ.conjugate());
+    }
+
+    private static void doTestEigenvalues(final MatrixStore<Double> matrix, final NumberContext accuracy, final double... expected) {
+
+        TestUtils.assertEquals(expected.length, matrix.getRowDim());
+        TestUtils.assertEquals(expected.length, matrix.getColDim());
+        TestUtils.assertTrue(matrix.isHermitian());
+
+        Eigenvalue<Double>[] evds = MatrixDecompositionTests.getPrimitiveEigenvalueSymmetric();
+        for (Eigenvalue<Double> evd : evds) {
+
+            evd.decompose(matrix);
+
+            Array1D<ComplexNumber> actual = evd.getEigenvalues();
+
+            if (DEBUG) {
+
+                BasicLogger.debug();
+                BasicLogger.debug("{}, ordered={}", evd.getClass(), evd.isOrdered());
+                BasicLogger.debug("Eigenvalues: {}", actual);
+            }
+
+            if (!evd.isOrdered()) {
+                actual.sort(Comparator.comparing(ComplexNumber::norm).reversed());
+            }
+
+            for (int i = 0; i < expected.length; i++) {
+                TestUtils.assertEquals(expected[i], actual.doubleValue(i), accuracy);
+                TestUtils.assertEquals(0.0, actual.get(i).i, accuracy);
+            }
+        }
+    }
+
+    private static void doTestEigenvaluesOfGenerated(final double... expected) {
+
+        MatrixStore<Double> generated = CaseEigenvalue.newRandom(expected);
+
+        CaseEigenvalue.doTestEigenvalues(generated, NumberContext.of(8), expected);
+    }
+
     private static void doVerifyGeneral(final Primitive64Store matrix) {
 
         for (Eigenvalue<Double> tmpEigenvalue : MatrixDecompositionTests.getPrimitiveEigenvalueGeneral()) {
@@ -97,7 +157,7 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
     @Test
     public void testComplexEigenpair() {
 
-        GenericStore<ComplexNumber> matrix = GenericStore.COMPLEX.make(2, 2);
+        GenericStore<ComplexNumber> matrix = GenericStore.C128.make(2, 2);
         matrix.set(0, 0, ComplexNumber.of(6.0, 0.0));
         matrix.set(1, 0, ComplexNumber.of(-2.0, -1.0));
         matrix.set(0, 1, ComplexNumber.of(-2.0, 1.0));
@@ -141,13 +201,13 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
     @Test
     public void testGeneralisedComplexEigenvalue() {
 
-        GenericStore<ComplexNumber> mtrxA = GenericStore.COMPLEX.make(2, 2);
+        GenericStore<ComplexNumber> mtrxA = GenericStore.C128.make(2, 2);
         mtrxA.set(0, 0, ComplexNumber.of(5.0, 0.0));
         mtrxA.set(1, 0, ComplexNumber.of(0.0, -3.0));
         mtrxA.set(0, 1, ComplexNumber.of(0.0, 3.0));
         mtrxA.set(1, 1, ComplexNumber.of(2.0, 0.0));
 
-        GenericStore<ComplexNumber> mtrxB = GenericStore.COMPLEX.make(2, 2);
+        GenericStore<ComplexNumber> mtrxB = GenericStore.C128.make(2, 2);
         mtrxB.set(0, 0, ComplexNumber.of(6.0, 0.0));
         mtrxB.set(1, 0, ComplexNumber.of(-2.0, -1.0));
         mtrxB.set(0, 1, ComplexNumber.of(-2.0, 1.0));
@@ -196,6 +256,26 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
         TestUtils.assertEquals(left, right);
     }
 
+    @Test
+    public void testGenerateRandomAllNegative() {
+        CaseEigenvalue.doTestEigenvaluesOfGenerated(-10, -5, -2, -1);
+    }
+
+    @Test
+    public void testGenerateRandomAllPositive() {
+        CaseEigenvalue.doTestEigenvaluesOfGenerated(10, 5, 2, 1);
+    }
+
+    @Test
+    public void testGenerateRandomSomeNegative() {
+        CaseEigenvalue.doTestEigenvaluesOfGenerated(10, 5, 2, -1);
+    }
+
+    @Test
+    public void testGenerateRandomSomeZero() {
+        CaseEigenvalue.doTestEigenvaluesOfGenerated(10, 5, 2, 0);
+    }
+
     /**
      * org.ojalgo.matrix.decomposition.EvD2D.hqr2 not converging #366. Following the debugger, it is the
      * method hqr2 that get stuck in a loop
@@ -277,7 +357,7 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
         ComplexNumber tmp33 = ComplexNumber.ZERO;
         ComplexNumber tmp44 = tmp33;
 
-        Array1D<ComplexNumber> tmpExpectedDiagonal = Array1D.C128.copy(tmp00, tmp11, tmp22, tmp33, tmp44);
+        Array1D<ComplexNumber> tmpExpectedDiagonal = Array1D.C128.copy(new ComplexNumber[] { tmp00, tmp11, tmp22, tmp33, tmp44 });
         NumberContext accuracyContext = NumberContext.of(7, 6);
 
         MatrixStore<Double> tmpRecreatedMatrix;
@@ -347,15 +427,15 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
             TestUtils.assertEquals(tmpSliceColumn.get(1).multiply(tmpFactor), tmpActual.get(1));
         }
 
-        GenericStore<ComplexNumber> tmpCmplA = GenericStore.COMPLEX.copy(tmpA);
-        GenericStore<ComplexNumber> tmpCmplD = GenericStore.COMPLEX.copy(tmpD);
-        GenericStore<ComplexNumber> tmpCmplV = GenericStore.COMPLEX.copy(tmpV);
+        GenericStore<ComplexNumber> tmpCmplA = GenericStore.C128.copy(tmpA);
+        GenericStore<ComplexNumber> tmpCmplD = GenericStore.C128.copy(tmpD);
+        GenericStore<ComplexNumber> tmpCmplV = GenericStore.C128.copy(tmpV);
 
         MatrixStore<ComplexNumber> tmpExp1 = tmpCmplA.multiply(tmpCmplV);
         MatrixStore<ComplexNumber> tmpAct1 = tmpCmplV.multiply(tmpCmplD);
         TestUtils.assertEquals(tmpExp1, tmpAct1);
 
-        GenericStore<ComplexNumber> tmpComplexD = GenericStore.COMPLEX.make(tmpLength, tmpLength);
+        GenericStore<ComplexNumber> tmpComplexD = GenericStore.C128.make(tmpLength, tmpLength);
         for (int j = 0; j < tmpLength; j++) {
             tmpComplexD.set(j, j, tmpValues.get(j));
         }
@@ -388,15 +468,15 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
         Array1D<ComplexNumber> tmpValues = tmpEvD.getEigenvalues();
         MatrixStore<ComplexNumber> tmpVectors = tmpEvD.getEigenvectors();
 
-        GenericStore<ComplexNumber> tmpCmplA = GenericStore.COMPLEX.copy(tmpA);
-        GenericStore<ComplexNumber> tmpCmplD = GenericStore.COMPLEX.copy(tmpD);
-        GenericStore<ComplexNumber> tmpCmplV = GenericStore.COMPLEX.copy(tmpV);
+        GenericStore<ComplexNumber> tmpCmplA = GenericStore.C128.copy(tmpA);
+        GenericStore<ComplexNumber> tmpCmplD = GenericStore.C128.copy(tmpD);
+        GenericStore<ComplexNumber> tmpCmplV = GenericStore.C128.copy(tmpV);
 
         MatrixStore<ComplexNumber> tmpExp1 = tmpCmplA.multiply(tmpCmplV);
         MatrixStore<ComplexNumber> tmpAct1 = tmpCmplV.multiply(tmpCmplD);
         TestUtils.assertEquals(tmpExp1, tmpAct1);
 
-        GenericStore<ComplexNumber> tmpAltD = GenericStore.COMPLEX.make(tmpLength, tmpLength);
+        GenericStore<ComplexNumber> tmpAltD = GenericStore.C128.make(tmpLength, tmpLength);
         MatrixStore<ComplexNumber> tmpAltV = tmpVectors;
 
         for (int j = 0; j < tmpLength; j++) {
@@ -614,4 +694,12 @@ public class CaseEigenvalue extends MatrixDecompositionTests {
         }
     }
 
+    @Test
+    public void testSpecial4by4() {
+
+        RawStore matrix = CaseLDL.newSpecialSchnabelEskow();
+
+        CaseEigenvalue.doTestEigenvalues(matrix, NumberContext.of(2), 8242.869, -0.378, -0.343, -0.248);
+    }
+
 }
