@@ -1,5 +1,5 @@
 /*
- * Copyright 1997-2022 Optimatika
+ * Copyright 1997-2024 Optimatika
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,10 +36,97 @@ import org.ojalgo.optimisation.Optimisation;
 import org.ojalgo.optimisation.Optimisation.Result;
 import org.ojalgo.optimisation.Optimisation.State;
 import org.ojalgo.optimisation.Variable;
+import org.ojalgo.optimisation.linear.LinearProblems;
 import org.ojalgo.type.context.NumberContext;
 
 public class IntegerProblems extends OptimisationIntegerTests {
 
+    /**
+     * <pre>
+     * OPTIMAL -97.59 @ { 0, 1, 0, 1, 0, 1, 1, 0 }
+     * ############################################
+     * 0 <= X0: 0 (-4836) <= 1
+     * 0 <= X1: 1 (-4824) <= 1
+     * 0 <= X2: 0 (1921) <= 1
+     * 0 <= X3: 1 (1929.9) <= 1
+     * 0 <= X4: 0 (2517) <= 1
+     * 0 <= X5: 1 (2526.51) <= 1
+     * 0 <= X6: 1 (270) <= 1
+     * 0 <= X7: 0 (267.5) <= 1
+     * 1 <= EXPR2: 1.0 <= 1
+     * 1 <= EXPR1: 1.0 <= 1
+     * 1 <= EXPR3: 1.0 <= 1
+     * 1 <= EXPR0: 1.0 <= 1
+     * ############################################
+     * </pre>
+     */
+    public static ExpressionsBasedModel makeModelGitHub513() {
+
+        ExpressionsBasedModel model = new ExpressionsBasedModel();
+
+        Variable varA1 = model.addVariable().weight(-4836.0).binary();
+        Variable varA2 = model.addVariable().weight(-4824.0).binary();
+        Variable varB1 = model.addVariable().weight(1921.0).binary();
+        Variable varB2 = model.addVariable().weight(1929.9).binary();
+        Variable varC1 = model.addVariable().weight(2517.0).binary();
+        Variable varC2 = model.addVariable().weight(2526.51).binary();
+        Variable varD1 = model.addVariable().weight(270.0).binary();
+        Variable varD2 = model.addVariable().weight(267.5).binary();
+
+        varA1.setValue(0);
+        varA2.setValue(1);
+        varB1.setValue(0);
+        varB2.setValue(1);
+        varC1.setValue(0);
+        varC2.setValue(1);
+        varD1.setValue(1);
+        varD2.setValue(0);
+
+        Expression constraintA = model.addExpression().level(1);
+        constraintA.set(varA1, 1);
+        constraintA.set(varA2, 1);
+        Expression constraintB = model.addExpression().level(1);
+        constraintB.set(varB1, 1);
+        constraintB.set(varB2, 1);
+        Expression constraintC = model.addExpression().level(1);
+        constraintC.set(varC1, 1);
+        constraintC.set(varC2, 1);
+        Expression constraintD = model.addExpression().level(1);
+        constraintD.set(varD1, 1);
+        constraintD.set(varD2, 1);
+
+        return model;
+    }
+
+    /**
+     * https://github.com/optimatika/ojAlgo/discussions/513
+     * <p>
+     * There was a case when the `IntegerSolver` returned an incorrect solution (constraint breaking) but
+     * reported it to be optimal. It was actually the `LinearSolver` that malfunctioned, but behaviour in the
+     * `IntegerSolver` that generated the problematic node model. Fixed this problem by 1) making sure the
+     * `LinearSolver` handles that case, and 2) altered the problematic behaviour in the `IntegerSolver` to be
+     * "safer".
+     *
+     * @see LinearProblems#testGitHub513()
+     */
+    @Test
+    public void testGitHub513() {
+
+        ExpressionsBasedModel model = IntegerProblems.makeModelGitHub513();
+
+        Result expected = Result.of(-97.59, State.OPTIMAL, 0, 1, 0, 1, 0, 1, 1, 0);
+
+        Result actual = model.maximise();
+
+        if (DEBUG) {
+            BasicLogger.debug(actual);
+            BasicLogger.debug(model);
+            model.validate(actual, BasicLogger.DEBUG);
+        }
+
+        TestUtils.assertStateAndSolution(expected, actual);
+    }
+
     /**
      * 20120227: Forgot to document what the problem was. Now I just check there is an optimal solution.
      */
@@ -71,25 +158,24 @@ public class IntegerProblems extends OptimisationIntegerTests {
     @Test
     public void testP20111010() {
 
-        Variable[] tmpVariables = new Variable[] { Variable.makeBinary("X").weight(ONE), Variable.makeBinary("Y").weight(ONE),
-                Variable.makeBinary("Z").weight(ONE) };
+        Variable[] tmpVariables = { Variable.makeBinary("X").weight(ONE), Variable.makeBinary("Y").weight(ONE), Variable.makeBinary("Z").weight(ONE) };
 
         ExpressionsBasedModel tmpModel = new ExpressionsBasedModel(tmpVariables);
 
-        Expression tmpC1 = tmpModel.addExpression("C1");
+        Expression tmpC1 = tmpModel.newExpression("C1");
         for (int i = 0; i < tmpModel.countVariables(); i++) {
             tmpC1.set(i, ONE);
         }
 
         tmpC1.level(ONE);
 
-        Expression tmpC2 = tmpModel.addExpression("C2");
+        Expression tmpC2 = tmpModel.newExpression("C2");
         for (int i = 0; i < tmpModel.countVariables(); i++) {
             tmpC2.set(i, ONE);
         }
         tmpC2.level(TWO);
 
-        Expression tmpC3 = tmpModel.addExpression("C3");
+        Expression tmpC3 = tmpModel.newExpression("C3");
         for (int i = 0; i < tmpModel.countVariables(); i++) {
             tmpC3.set(i, ONE);
         }
@@ -129,24 +215,24 @@ public class IntegerProblems extends OptimisationIntegerTests {
     @Test
     public void testP20130409a() {
 
-        Variable[] variables = new Variable[] { new Variable("x1").lower(BigMath.ZERO).weight(BigMath.ONE), new Variable("x2013").lower(BigMath.ZERO).integer(),
+        Variable[] variables = { new Variable("x1").lower(BigMath.ZERO).weight(BigMath.ONE), new Variable("x2013").lower(BigMath.ZERO).integer(),
                 new Variable("x2014").lower(BigMath.ZERO).integer() };
 
         ExpressionsBasedModel model = new ExpressionsBasedModel(variables);
 
-        Expression expr1 = model.addExpression("Expr1");
+        Expression expr1 = model.newExpression("Expr1");
         expr1.set(0, -1);
         expr1.set(1, 5100);
         expr1.set(2, -5000);
         expr1.upper(BigMath.ZERO);
 
-        Expression expr2 = model.addExpression("Expr2");
+        Expression expr2 = model.newExpression("Expr2");
         expr2.set(0, 1);
         expr2.set(1, 5100);
         expr2.set(2, -5000);
         expr2.lower(BigMath.ZERO);
 
-        Expression expr3 = model.addExpression("Expr3");
+        Expression expr3 = model.newExpression("Expr3");
         expr3.set(1, 5000);
         expr3.set(2, 5000);
         expr3.level(new BigDecimal(19105000));
@@ -180,41 +266,41 @@ public class IntegerProblems extends OptimisationIntegerTests {
         model.addVariable(x2014);
         model.addVariable(x2015);
 
-        Expression obj = model.addExpression("obj");
+        Expression obj = model.newExpression("obj");
         obj.set(x1, 1);
         obj.weight(1);
 
-        Expression c1 = model.addExpression("c1");
+        Expression c1 = model.newExpression("c1");
         c1.set(x1, 1);
         c1.lower(0);
 
-        Expression c2 = model.addExpression("c2");
+        Expression c2 = model.newExpression("c2");
         c2.set(x2014, -5000);
         c2.set(x2013, 5100);
         c2.set(x1, -1);
         c2.upper(0);
 
-        Expression c3 = model.addExpression("c3");
+        Expression c3 = model.newExpression("c3");
         c3.set(x2014, -5000);
         c3.set(x2013, 5100);
         c3.set(x1, 1);
         c3.lower(0);
 
-        Expression c4 = model.addExpression("c4");
+        Expression c4 = model.newExpression("c4");
         c4.set(x2014, 150);
         c4.set(x2013, 5100);
         c4.set(x2015, -5000);
         c4.set(x1, -1);
         c4.upper(0);
 
-        Expression c5 = model.addExpression("c5");
+        Expression c5 = model.newExpression("c5");
         c5.set(x2014, 150);
         c5.set(x2013, 5100);
         c5.set(x2015, -5000);
         c5.set(x1, 1);
         c5.lower(0);
 
-        Expression c6 = model.addExpression("c6");
+        Expression c6 = model.newExpression("c6");
         c6.set(x2015, 5000);
         c6.set(x2014, 5000);
         c6.set(x2013, 5000);
@@ -309,7 +395,7 @@ public class IntegerProblems extends OptimisationIntegerTests {
      * </p>
      */
     @Test
-    public void testP20160701() {
+    public void testSimpleTSP20160701() {
 
         int n = 6;
         double[][] c = new double[n][n];
@@ -356,14 +442,12 @@ public class IntegerProblems extends OptimisationIntegerTests {
         Variable[][] x = new Variable[n][n];
         for (int i = 0; i < n; i++) {
             for (int j = 0; j < n; j++) {
-                x[i][j] = Variable.make("x" + i + "_" + j).binary().weight(c[i][j]);
-                model.addVariable(x[i][j]);
+                x[i][j] = model.newVariable("x" + i + "_" + j).binary().weight(c[i][j]);
             }
         }
         Variable[] u = new Variable[n];
         for (int i = 1; i < n; i++) {
-            u[i] = new Variable("u" + i);
-            model.addVariable(u[i]);
+            u[i] = model.newVariable("u" + i);
         }
 
         //CONSTRAINTS
@@ -371,7 +455,7 @@ public class IntegerProblems extends OptimisationIntegerTests {
         //flow_out:
         //sum(j in cities : i!=j) x[i][j]==1;
         for (int i = 0; i < n; i++) {
-            Expression constraint_line = model.addExpression("constraint_line" + i).lower(1).upper(1);
+            Expression constraint_line = model.newExpression("constraint_line_" + i).lower(1).upper(1);
             for (int j = 0; j < n; j++) {
                 if (i != j) {
                     constraint_line.set(x[i][j], 1);
@@ -383,7 +467,7 @@ public class IntegerProblems extends OptimisationIntegerTests {
         //flow_in:
         //sum(i in cities : i!=j) x[i][j]==1;
         for (int j = 0; j < n; j++) {
-            Expression constraint_column = model.addExpression("constraint_column" + j).lower(1).upper(1);
+            Expression constraint_column = model.newExpression("constraint_column_" + j).lower(1).upper(1);
             for (int i = 0; i < n; i++) {
                 if (i != j) {
                     constraint_column.set(x[i][j], 1);
@@ -397,7 +481,7 @@ public class IntegerProblems extends OptimisationIntegerTests {
         for (int i = 1; i < n; i++) {
             for (int j = 1; j < n; j++) {
                 if (i != j) {
-                    Expression constraint_subroute = model.addExpression("constraint_subroute" + i + "_" + j).upper(n - 1);
+                    Expression constraint_subroute = model.newExpression("constraint_subroute_" + i + "_" + j).upper(n - 1);
                     constraint_subroute.set(u[i], 1);
                     constraint_subroute.set(u[j], -1);
                     constraint_subroute.set(x[i][j], n);
@@ -405,7 +489,9 @@ public class IntegerProblems extends OptimisationIntegerTests {
             }
         }
 
-        // model.options.debug(IntegerSolver.class);
+        if (OptimisationIntegerTests.DEBUG) {
+            model.options.debug(IntegerSolver.class);
+        }
 
         Optimisation.Result result = model.minimise();
 
@@ -430,7 +516,8 @@ public class IntegerProblems extends OptimisationIntegerTests {
         }
 
         TestUtils.assertStateNotLessThanOptimal(result);
-        TestUtils.assertEquals(917.3134949394164, result.getValue());
+        TestUtils.assertTrue(model.validate(result));
+        TestUtils.assertEquals(917.31349493942, result.getValue());
     }
 
     /**
@@ -442,14 +529,14 @@ public class IntegerProblems extends OptimisationIntegerTests {
         NumberContext precision = NumberContext.of(14, 12);
         ExpressionsBasedModel model = new ExpressionsBasedModel();
 
-        Variable varX = model.addVariable("X").binary();
-        Variable varY = model.addVariable("Y").binary();
+        Variable varX = model.newVariable("X").binary();
+        Variable varY = model.newVariable("Y").binary();
 
         // (X <= Y * 0.5)  equivalent to  (0 <= Y * 0.5 - X)
         // only one solution: X = 0, Y = 1
-        model.addExpression("X < Y * 0.5").lower(ZERO).set(varX, NEG).set(varY, HALF);
+        model.newExpression("X < Y * 0.5").lower(ZERO).set(varX, NEG).set(varY, HALF);
         // objective function
-        model.addExpression("sum").weight(ONE).set(varX, ONE).set(varY, ONE);
+        model.newExpression("sum").weight(ONE).set(varX, ONE).set(varY, ONE);
 
         if (DEBUG) {
             BasicLogger.debug("Original 1 model");
@@ -492,14 +579,14 @@ public class IntegerProblems extends OptimisationIntegerTests {
         // model.options.debug(Optimisation.Solver.class);
         model.options.iterations_abort = 2;
 
-        Variable varX = model.addVariable("X").binary();
-        Variable varY = model.addVariable("Y").binary();
+        Variable varX = model.newVariable("X").binary();
+        Variable varY = model.newVariable("Y").binary();
 
         // two possible solutions: X = 0, Y = 1 or X = 1, Y = 0
-        Expression constraint = model.addExpression("X + Y <= 1").upper(ONE).set(varX, ONE).set(varY, ONE);
+        Expression constraint = model.newExpression("X + Y <= 1").upper(ONE).set(varX, ONE).set(varY, ONE);
 
         // function maximized by X = 1
-        Expression objectiveFunction = model.addExpression("X").weight(ONE).set(varX, ONE);
+        Expression objectiveFunction = model.newExpression("X").weight(ONE).set(varX, ONE);
 
         Result result = model.maximise();
 
